# Nextcloud App Sign Live! Administrators Guide - v.2022-01-29

## Overview
The Nextcloud App Sign Live! enhances Nextcloud's collaborative functions with the ability to sign and seal documents with different stores and certificate service providers. Your documents remain under your control.

The connection to the different certificate service providers and certificate stores is given by [Sign Live! cloud suite gears][11]. Hence you need access to a running gears instance, preferably hosted in your environment but also possible in the cloud, e. g. the intarsys gears demo instance.

![](pics/default.png)

Dependend on the device to be used, you have to meet some additional prerequisites. In case additional licenses are needed contact [intarsys sales][4] for more information.

### Swisscom AIS Seal

With the demo configuration this device lets the user create an advanced remote demo seal. 

To create legally binding seals you have to meet the following preconditions:
- a licensed gears installation
- a licensed access to a Swisscom AIS instance

### Swisscom AIS Personal Signature

Before creating a personal signature with Swisscom AIS every user should perform some preliminary steps (see [Users Guide][3]). 

With the Nextcloud demo configuration this device lets the user authorize and create an advanced personal remote demo signature.

To create legally binding signatures you have to meet the following preconditions:
- a licensed gears installation
- a licensed access to a Swisscom AIS instance
- every signer has to be identified by Swisscom.

### D-Trust sign-me Personal Signature

Before creating a personal signature with sign-me every user should perform some preliminary steps (see [Users Guide][3]). The user is identified automatically during the first signing process. The Identification process for the sign-me demo instance is free of charge and can only be used for the demo instance. It does not interfere with identifications for productive sign-me instances.

With the intarsys Nextcloud demo instance this device lets the user authorize and create an advanced personal remote demo signature.

To create legally binding signatures in your environment you have to meet the following preconditions:
- a licensed gears installation
- a licensed access to a sign-me instance

### D-Trust qSeal-ID

With the Nextcloud demo configuration this device lets the user create an advanced personal remote demo seal.

To create legally binding seals you have to meet the following preconditions:
- a licensed gears installation
- replace the demo certificate in your installation by a licensed qSeal-ID certificate

### Local Certificates (Smartcard, Tokens, Certificate Store)

This device offers signatures with certificates available at your user's workplace. Every user has to install [Sign Live! cloud suite bridge][7] to get access to 
- Smartcards in smartcard readers
- PKCS11 Tokens connected via USB
- Certificates stored in the Windows Certificate Store

The bridge is free of charge for demo purposes or in connection with a licensed gears installation. If there is no certificate already available on the client, bridge provides the user with a demo certificate (dependend on the configuration).

To get a licensed installation you have to meet the following preconditions:
- a licensed gears/bridge installation
- a certificate on the client
  - smartcard & card reader
  - PKCS11-Token
  - soft certificate in Windows certificate store

## Installation 

### Nextcloud App Sign Live! 
Install the app from the [Nextcloud App Store][1] into your Nextcloud instance. If you don’t have a working Nextcloud instance, use [intarsys Nextcloud demo instance][9].

### Sign Live! cloud suite gears
The Nextcloud App Sign Live! is preconfigured to work with the gears demo instance. If you prefere to install gears in your environment contact [intarsys sales][4] to get access to the actual gears download, download and have a look at the gears installation process described in the [gears users manual][8].

### Sign Live! cloud suite bridge
Contact [intarsys sales][4] to get access to the actual [bridge download][7], download and start the installer on every client, where signing with workplace devices is needed.

## Configuration
The Sign Live! app's configuration is available via the Nextcloud settings. The administrator's part is available in the Administration Settings ***Sign Live!*** for administrators only, the uers's part ist available in the Personal Settings ***Sign Live!*** for every user individually.

![](pics/settings_admin.png)

### Connection to gears

The default configuration connects to the [intarsys gears demo instance][10]. To change it, enter the URL of your gears instance.

### Profiles

The Nextcloud user initiates a signature process via one or more items in the file's context menu. Each item is defined by a **Signature Profile**. **Default Profiles** - marked as ***default*** -  will be available for all users if and only if the checkbox ***show default profiles in context menu*** is active. All other profiles – **Group Profiles** - have to be linked to a user group to be available for the members of this group.

The ***Profile name*** will be shown in the context menu, the position is defined by the profile's ***Priority*** and alphabetical sorting of its name.

A profile is defined by a ***Profile type*** and a ***JSON section***. The profile type indicates, whether the document is signed directly - ***Signer*** - or only after having seen it in a viewer - ***Viewer***.

The JSON section gives you the complete flexibility of gears. For normal cases the default profiles should work. If not, contact [intarsys support][5].

![](pics/settings_profile.png)

For every signature request the JSON section will be enhanced with Nextcloud settings via **Variables**. There are **System-Wide Variables** - actually only "company name" - to be edited in the Nextcloud Administrator's part of the settings and **User Specific Variables** to be edited in the Nextcloud personal settings.  Press on the label ***JSON section i*** to get the list of the variables available.

> Tip: If you change a profile, don't forget to ***Save Changes***.

> Tip: For remote signature scenarios personal settings are needed.

> Tip: For bridge scenarios set the certificate filters appropriately.

### JSON Section

For the Nextcloud demo scenario the JSON section of type *Viewer* looks like this (enclose it in  `{}`): 

	"variables": {
		"bridge.principalFilter": "true",
		"bridge.deviceProviderInclude": "demo;pkcs11;smartcard;windows",
		"signatureImage": "${sl.encodedsignatureimage}"
	},
	"options": {
		"redirectUri": "${sl.system.redirect_url}",
		"principal": {
			"name": "${nc.displayname}",
			"claims": {
				"msisdn": "${sl.msisdn}",
				"givenname": "${sl.givenname}",
				"surname": "${sl.surname}",
				"c": "${sl.country}",
				"signMeUsername": "${sl.signmeusername}"
			}
		}
	},
	"configuration": "ncSignature",
	${sl.system.documents_section}

## Typical Adminstrative Tasks

### Connection Test

Open in a internet browser on the Nextcloud Server the ***Server Address / Server URI*** or use curl:

 `curl https://cloudsuite.intarsys.de/cloudsuite-gears-nextcloud/core/`
 
 The answer should be a valid HTML page.
 
### Check the Log
The Log is available via Settings>Administration Logging.
You need administrator rights.


---
## Contact
- [intarsys sales][4]
- [intarsys support][5] 

## Links

[1]: https://apps.nextcloud.com/ "Nextcloud App Store"
[2]: https://www.intarsys.de/node/1283/attachment "Nextcloud App Sign Live! Adminstrators Guide"
[3]: https://www.intarsys.de/node/1284/attachment "Nextcloud App Sign Live! Users Guide"
[4]: https://www.intarsys.de/contact/sales "intarsys sales"
[5]: https://www.intarsys.de/contact/support "intarsys support"
[6]: https://www.intarsys.de/versionshinweise "intarsys Sign Live! cloud suite gears"
[7]: https://www.intarsys.de/versionshinweise "intarsys Sign Live! cloud suite bridge"
[8]: https://cloudsuite.intarsys.de/cloudsuite-gears/core/doc/Sign%20Live%20cloud%20suite%20gears%20manual.pdf "intarsys Sign Live! cloud suite gears manual"
[9]: https://nextclouddemo.intarsys.de/ "intarsys Nextcloud demo instance"
[10]: https://cloudsuite.intarsys.de/cloudsuite-gears-nextcloud/core/ "intarsys gears demo instance"
[11]: https://www.intarsys.de/produkte/sign-live-cloud-suite-gears "intarsys Sign Live! cloud suite gears"
