$(document).ready(function() {

	var cloudsuite_gears_signature_profile_count = 0;
	var cloudsuite_gears_all_profiles = null;
	var cloudsuite_gears_nextcloud_groups = null;

	var profile_default_menu_priority_value = 75;

	var default_profile_option_value = false;

	function saveSettings() {
        console.log("saveSettings() for admin settings in signlive called.");

		OC.msg.startSaving('#cloudsuite_gears_settings_notifications_msg');

		var post_1 = $("#cloudsuite_gears_general_settings_form input[type='text']").map(function() { return this.name+"="+this.value;}).get().join("&");
		var post_2 = $("#cloudsuite_gears_general_settings_form input[type='checkbox']").map(function() { return this.name+"="+this.checked;}).get().join("&");

		if (post_1 != "" && post_2 != "") {
			post_1 += "&" + post_2;
		} else {
			post_1 += post_2;
		}

        console.log("Values to POST: " + post_1);
        
		$.post(OC.generateUrl('/apps/signlive/settings/admin'), post_1, function(response) {
			OC.msg.finishedSuccess('#cloudsuite_gears_settings_notifications_msg', response.data.message);
            console.debug("saveSettings() for admin settings in signlive succeeded: " + response.data.message);
		});
        console.debug("saveSettings() for admin settings in signlive FINISHED.");
	}

	// Helper functions for hiding controls and help text
	function toggleHideElement(element_name, hidden_value) {
		console.debug('Show/hide element: ' + element_name);
		var element_to_hide = document.getElementById(element_name);
		if (element_to_hide !== undefined && element_to_hide !== null) {
			console.debug('Element found: ' + element_name + ', current display style: \'' + element_to_hide.style.display + '\'');
			if (hidden_value === false) {
				console.debug("Showing it.");
				element_to_hide.style.display = "block";
			} else if (hidden_value === true) {
				console.debug("Hiding it.");
				element_to_hide.style.display = "none";
			}
		} else {
			console.debug('Element to hide NOT FOUND: ' + element_name + '!');
		}
	}

	// Helper functions for enabling/disabling controls
	function toggleEnableElement(element_name, enabled_value) {
		console.debug('Disable/enable element: ' + element_name);
		var element_to_hide = document.getElementById(element_name);
		if (element_to_hide !== undefined && element_to_hide !== null) {
			console.debug('Element found: ' + element_name + ', disabled set to: \'' + element_to_hide.disabled + '\'');
			if (enabled_value === false) {
				console.debug("Disable it.");
				element_to_hide.disabled = true;
			} else if (enabled_value === true) {
				console.debug("Enable it.");
				element_to_hide.disabled = false;
			}
		} else {
			console.debug('Element to enable/disable NOT FOUND: ' + element_name + '!');
		}
	}

	// Helper function to clear status message element
	function clearProfileStatusMessage() {
		var status_span = document.getElementById('cloudsuite_gears_signature_profiles_notifications_msg');
		status_span.innerText = '';
	}

	function repopulateProfileGroupLists(profile_id) {
		if (cloudsuite_gears_all_profiles[profile_id]['profile_type'] === 'viewer_based_signature') {
			document.getElementById('cloudsuite_gears_user_profile_type').selectedIndex = 0;
		} else if (cloudsuite_gears_all_profiles[profile_id]['profile_type'] === 'automatic_signature') {
			document.getElementById('cloudsuite_gears_user_profile_type').selectedIndex = 1;
		} else {
			document.getElementById('cloudsuite_gears_user_profile_type').selectedIndex = -1;
		}
		var all_groups_options = document.querySelectorAll('#cloudsuite_gears_signature_all_nextcloud_groups option');
		all_groups_options.forEach(o => o.remove());

		console.log('Options count in all groups list after clearing: ' + all_groups_options.length);

		var profile_groups_options = document.querySelectorAll('#cloudsuite_gears_signature_profile_nextcloud_groups option');
		profile_groups_options.forEach(o => o.remove());

		console.log('Options count in profile groups list after clearing: ' + profile_groups_options.length);

		if (cloudsuite_gears_nextcloud_groups != null) {
			// Then first repopulate the list of all Nextcloud groups
			var ng_group_names = cloudsuite_gears_nextcloud_groups['nextcloud_group_names'];
			for (const group in ng_group_names) {
				var group_entry = document.createElement('option');
				group_entry.value = group;
				group_entry.text = ng_group_names[group];
				document.getElementById('cloudsuite_gears_signature_all_nextcloud_groups').options.add(group_entry);
			}
			// And the list of valid groups for the currently selected profile
			// List of Nextcloud groups selected for the active profile are stored as a ";;"-separated string value
			var profile_groups = cloudsuite_gears_all_profiles[profile_id]['profile_nextcloud_groups'];
			console.log('Nextcloud groups for profile ' + profile_id + ': ' + profile_groups);
			if (profile_groups !== null && profile_groups !== undefined) {
				var profile_group_names = profile_groups.split(';;');
				if (profile_group_names.length > 0) {
					for (const group_name in profile_group_names) {
						var group_entry = document.createElement('option');
						group_entry.value = profile_group_names[group_name];
						group_entry.text = profile_group_names[group_name];
						document.getElementById('cloudsuite_gears_signature_profile_nextcloud_groups').options.add(group_entry);
					}
				}
			}
		}
	}

	function updateSignatureProfileForm(profile_id) {
		console.debug('updateSignatureProfileForm(\'' + profile_id + '\')')
		
		var given_profile_numeric_id = -1;
		var given_profile_id_parts = profile_id.split(';;');
		if (given_profile_id_parts.length === 2) {
			given_profile_numeric_id = given_profile_id_parts[0];
			given_profile_numeric_id = given_profile_numeric_id.replace('cloudsuite_gears_profile_id_', '');
			given_profile_numeric_id = parseInt(given_profile_numeric_id);
			console.debug('Numeric ID of given profile_id \'' + profile_id + '\': ' + given_profile_numeric_id);
		} else {
			console.error('Failed to extract numeric ID from given profile_id \'' + profile_id + '\'! Can not update profile properties form.');
			return;
		}

		if (cloudsuite_gears_all_profiles === null || cloudsuite_gears_all_profiles === undefined) {
			console.debug('cloudsuite_gears_all_profiles is not defined or not valid, querying signature profile data from backend.');
			queryProfileSelectionListFromBackend();
		}

		var profile_id_for_form_update = '';
		for (profile_id in cloudsuite_gears_all_profiles) {
			var profile_numeric_id = -1;
			var profile_id_parts = profile_id.split(';;');
			if (profile_id_parts.length === 2) {
				profile_numeric_id = profile_id_parts[0];
				profile_numeric_id = profile_numeric_id.replace('cloudsuite_gears_profile_id_', '');
				profile_numeric_id = parseInt(profile_numeric_id);
				console.debug(' * Comparing profile IDs: ' + profile_numeric_id + ' == ' + given_profile_numeric_id);
				if (given_profile_numeric_id === profile_numeric_id) {
					profile_id_for_form_update = profile_id;
					console.debug('Found matching numeric profile ID in list of all signature profiles: ' + profile_numeric_id + ' matching profile ID: ' + profile_id_for_form_update);
					break;
				}
			}
		}

		if (profile_id_for_form_update === '') {
			console.error('Failed to find entry matching profile ID \'' + profile_id + '\' in list of available signature profiles, can not update profile properties form!');
			return;
		}

		default_profile_option_value = false;
		if (cloudsuite_gears_all_profiles[profile_id_for_form_update] === null || cloudsuite_gears_all_profiles[profile_id_for_form_update] === undefined) {
			console.error('Profile data for profile_id \'' + profile_id_for_form_update + '\' not found in profile data! Can not update profile properties form.');
			console.error('Available profiles: ');
			for (profile_id in cloudsuite_gears_all_profiles) {
				console.error(' * ' + profile_id);
			}
			return;
		} else {
			var new_profile_is_default_profile = cloudsuite_gears_all_profiles[profile_id_for_form_update]['profile_default_profile'];
			console.debug('new_profile_is_default_profile = ' + new_profile_is_default_profile);
			
			document.getElementById('cloudsuite_gears_user_profile_name').value = cloudsuite_gears_all_profiles[profile_id_for_form_update]['profile_name'];
			if (cloudsuite_gears_all_profiles[profile_id_for_form_update]['profile_default_profile'] === true || 
				cloudsuite_gears_all_profiles[profile_id_for_form_update]['profile_default_profile'] === 'true' ||
				cloudsuite_gears_all_profiles[profile_id_for_form_update]['profile_default_profile'] === 1 ||
				parseInt(cloudsuite_gears_all_profiles[profile_id_for_form_update]['profile_default_profile']) === 1) {
				console.debug('Check-ing checkbox for default profile option.');
				document.getElementById('cloudsuite_gears_signature_default_profile').checked = true;
				default_profile_option_value = true;
			} else {
				console.debug('NOT check-ing checkbox for default profile option.');
				document.getElementById('cloudsuite_gears_signature_default_profile').checked = false;
				default_profile_option_value = false;
			}

			if (default_profile_option_value === true) {
				console.debug('DISABLING (disabled = true, enabled_value = false) Nextcloud group selection controls');
				toggleEnableElement('cloudsuite_gears_signature_all_nextcloud_groups', false);
				toggleEnableElement('cloudsuite_gears_signature_profile_nextcloud_groups', false);
				toggleEnableElement('cloudsuite_gears_signature_add_groups', false);
				toggleEnableElement('cloudsuite_gears_signature_remove_groups', false);
				toggleEnableElement('cloudsuite_gears_signature_nextcloud_groups_label', false);
			}
			else if (default_profile_option_value === false) {
				console.debug('ENABLING (disabled = false, enabled_value = true) Nextcloud group selection controls');
				toggleEnableElement('cloudsuite_gears_signature_all_nextcloud_groups', true);
				toggleEnableElement('cloudsuite_gears_signature_profile_nextcloud_groups', true);
				toggleEnableElement('cloudsuite_gears_signature_add_groups', true);
				toggleEnableElement('cloudsuite_gears_signature_remove_groups', true);
				toggleEnableElement('cloudsuite_gears_signature_nextcloud_groups_label', true);
			}

			if (cloudsuite_gears_all_profiles[profile_id_for_form_update]['profile_menu_priority'] !== null && cloudsuite_gears_all_profiles[profile_id_for_form_update]['profile_menu_priority'] !== undefined) {
				document.getElementById('cloudsuite_gears_signature_profile_menu_priority').value = cloudsuite_gears_all_profiles[profile_id_for_form_update]['profile_menu_priority'];
			} else {
				document.getElementById('cloudsuite_gears_signature_profile_menu_priority').value = menu_entry_priority_value;
			}

			console.debug('Profile type: ' + cloudsuite_gears_all_profiles[profile_id_for_form_update]['profile_type']);
			if (cloudsuite_gears_all_profiles[profile_id_for_form_update]['profile_type'] === 'viewer_based_signature') {
				console.debug('Setting \'Viewer\' option as displayed.');
				document.getElementById('cloudsuite_gears_user_profile_type').selectedIndex = 0;
			} else if (cloudsuite_gears_all_profiles[profile_id_for_form_update]['profile_type'] === 'automatic_signature') {
				console.debug('Setting \'Signer\' option as displayed.');
				document.getElementById('cloudsuite_gears_user_profile_type').selectedIndex = 1;
			} else {
				console.error('Invalid profile type option -- signature profile settings are corrupt.');
				document.getElementById('cloudsuite_gears_user_profile_type').selectedIndex = -1;
			}

			document.getElementById('cloudsuite_gears_signature_profile_json').value = cloudsuite_gears_all_profiles[profile_id_for_form_update]['profile_json_section'];
		
			repopulateProfileGroupLists(profile_id_for_form_update);
		}
	}

	// Helper function for sorting signature profiles by display name
	function sortProfileList() {
		var sorted_profile_names_and_id_data = {};

		// Early bail-out in case there is no valid profile data
		if (cloudsuite_gears_all_profiles === null ||
			cloudsuite_gears_all_profiles === undefined) {
			return sorted_profile_names_and_id_data;
		}
		
		var sorted_profile_names = [];
		var profile_names_and_ids = {};
		for (var profile_id in cloudsuite_gears_all_profiles) {
			// Filter out reserved values included in the profiles list 
			if (profile_id === 'menu_entry_priority_value' ||
				profile_id === 'relative_app_path') {
				continue;
			}

			sorted_profile_names.push(cloudsuite_gears_all_profiles[profile_id]['profile_name']);
			profile_names_and_ids[cloudsuite_gears_all_profiles[profile_id]['profile_name']] = profile_id;
		}
		sorted_profile_names = sorted_profile_names.sort();

		console.debug('=== Sorted profile names ===');
		for (var k = 0; k < sorted_profile_names.length; k++) {
			console.debug(' - ' + sorted_profile_names[k]);
		}

		sorted_profile_names_and_id_data['sorted_profile_names'] = sorted_profile_names;
		sorted_profile_names_and_id_data['profile_names_and_ids'] = profile_names_and_ids;

		return sorted_profile_names_and_id_data;
	}

	// Helper function to determine the position of a new profile name entry in the profile selection list
	function getPositionOfNewProfileEntry(sorted_profile_data, profile_name, old_profile_name, add_new_profile_entry) {
		console.debug('getPositionOfNewProfileEntry(\'' + profile_name + '\', \'' + old_profile_name + '\', ' + add_new_profile_entry + ')');

		if (('sorted_profile_names' in sorted_profile_data) &&
			('profile_names_and_ids' in sorted_profile_data)) {
			var new_profile_index = -1;
			
			if (add_new_profile_entry === true) {
				console.debug('Looking for index to insert new profile name entry at: ' + profile_name);
				// Append the new profile's name to a copy of the existing sorted list of profile names, then see which index it ends up at
				var current_sorted_profile_names = sorted_profile_data['sorted_profile_names'];
				current_sorted_profile_names.push(profile_name);
				
				current_sorted_profile_names = current_sorted_profile_names.sort();

				for (var k = 0; k < current_sorted_profile_names.length; k++)
				{
					console.debug('Comparing profile names: ' + current_sorted_profile_names[k] + ' == ' + profile_name);
					if (current_sorted_profile_names[k] === profile_name) {
						console.debug('Found new profile name index: ' + k);
						new_profile_index = k;
						break;
					}
				}
			
				return new_profile_index;
			} else {
				// In case of renaming an existing profile the index of old_profile_name needs to be found
				// for subsequently replacing it with new_profile_name
				console.debug('Looking for index to insert to replace existing name entry ' + old_profile_name + ' with new name entry ' + profile_name + ' at.');
				var current_sorted_profile_names = sorted_profile_data['sorted_profile_names'];
				for (var k = 0; k < current_sorted_profile_names.length; k++)
				{
					console.debug('Comparing profile names: ' + current_sorted_profile_names[k] + ' == ' + profile_name);
					if (current_sorted_profile_names[k] === old_profile_name) {
						console.debug('Found new profile name index: ' + k);
						new_profile_index = k;
						break;
					}
				}

				return new_profile_index;
			}
		}
		return -1;
	}

	// index_to_select refers to list index in sorted list of profile display names
	function updateProfileSelectionList(index_to_select = -1) {
		var profile_list_options = document.querySelectorAll('#profile_select option');
		profile_list_options.forEach(o => o.remove());
		
		var sorted_profile_data = sortProfileList();
		
		if (cloudsuite_gears_all_profiles !== null) {
			for (var k = 0; k < sorted_profile_data['sorted_profile_names'].length; k++) {
				var profile_name = sorted_profile_data['sorted_profile_names'][k];
				var profile_id = sorted_profile_data['profile_names_and_ids'][profile_name];
				if (cloudsuite_gears_all_profiles[profile_id] !== null && cloudsuite_gears_all_profiles[profile_id] !== undefined) {
					var profile_entry = document.createElement('option');
					profile_entry.value = profile_id;
					console.debug('Adding profile to selection list: ' + profile_name);
					profile_entry.text = profile_name;
					document.getElementById('profile_select').options.add(profile_entry);
				} else {
					console.error('Encountered an invalid signature profile entry when updating profile list! Profile ID: ' + profile_id);
				}
			}

			// At this point the selection list for profiles has been re-populated.
			// The new index to select can be applied now (if one has been explicitly given)
			console.debug('New index to select in profile selection dropdown: ' + index_to_select);
			if (index_to_select >= 0 && index_to_select < sorted_profile_data['sorted_profile_names'].length) {
				document.getElementById('profile_select').selectedIndex = index_to_select;
			}
		}
	}

	function queryProfileSelectionListFromBackend() {
		cloudsuite_gears_signature_profile_count = 0;
		$.ajax({
			url: OC.generateUrl('/apps/signlive/sign_document/get_all_signature_profiles'),
			type: 'post',
			async : false,
			success: function(response) {
				console.debug('=== Received signature profile list from backend in queryProfileSelectionListFromBackend() ===')
				cloudsuite_gears_signature_profile_count = 0;
				cloudsuite_gears_all_profiles = response;
				if (cloudsuite_gears_all_profiles !== null) {
					for (var profile_id in cloudsuite_gears_all_profiles) {
						// Filter out reserved values included in the profiles list 
						if (profile_id === 'menu_entry_priority_value' ||
						    profile_id === 'relative_app_path') {
							continue;
						}

						var profile_unique_id = profile_id.split(';;')[0];
						profile_unique_id = profile_unique_id.replace('cloudsuite_gears_profile_id_', '');
						var profile_numeric_id = parseInt(profile_unique_id);
						console.debug('Signature profile ' + cloudsuite_gears_signature_profile_count + ' with profile ID: ' + profile_id + ' and numeric ID: ' + profile_numeric_id);
						cloudsuite_gears_signature_profile_count += 1;
					}

					profile_default_menu_priority_value = parseInt(cloudsuite_gears_all_profiles['menu_entry_priority_value']);

					console.debug('Signature profile count in Nextcloud settings: ' + cloudsuite_gears_signature_profile_count);
				}
			},
			error: function(response) {
				console.error("Failed to query list of defined signature profiles! Profile properties will not be editable.");
				cloudsuite_gears_signature_profile_count = 0;
			}
		});
	}

	function queryAllNextcloudGroupsFromBackend() {
		$.ajax({
			url: OC.generateUrl('/apps/signlive/sign_document/get_all_nextcloud_group_names'),
			type: 'post',
			async : false, // OK on document.ready()?
			success: function(response) {
				console.log("Retrieved list of Nextcloud user groups: " + response['nextcloud_group_names']);
				cloudsuite_gears_nextcloud_groups = response;
			},
			error: function(response) {
				console.error("Failed to query list of Nextcloud user groups! Profile properties will not be completely editable.");
			}
		});
	}

	function getSelectedProfileData() {
		var profile_data = [];
		profile_data['cloudsuite_gears_profile_id'] = document.getElementById('profile_select').options[document.getElementById('profile_select').options.selectedIndex].value;
		profile_data['cloudsuite_gears_profile_name'] = document.getElementById('cloudsuite_gears_user_profile_name').value;
		profile_data['cloudsuite_gears_profile_type'] = document.getElementById('cloudsuite_gears_user_profile_type').value;
		if (document.getElementById('cloudsuite_gears_signature_default_profile').checked) {
			profile_data['cloudsuite_gears_default_signature_profile'] = 1;
		} else {
			profile_data['cloudsuite_gears_default_signature_profile'] = 0;	
		}
		profile_data['cloudsuite_gears_profile_json'] = document.getElementById('cloudsuite_gears_signature_profile_json').value;

		var profile_menu_priority = document.getElementById('cloudsuite_gears_signature_profile_menu_priority').value;
		if (profile_menu_priority === null || profile_menu_priority === undefined || profile_menu_priority === '') {
			profile_menu_priority = menu_entry_priority_value;
		}

		profile_data['cloudsuite_gears_signature_profile_menu_priority'] = profile_menu_priority;
		// Assemble list of selected Nextcloud groups as ";;"-separated string value
		var num_nc_groups = document.getElementById('cloudsuite_gears_signature_profile_nextcloud_groups').options.length;
		if (num_nc_groups > 0) {
			var nc_group_list = '';
			for (var i = 0; i < num_nc_groups - 1; i++) {
				nc_group_list += document.getElementById('cloudsuite_gears_signature_profile_nextcloud_groups').options[i].value + ';;';
			}
			nc_group_list += document.getElementById('cloudsuite_gears_signature_profile_nextcloud_groups').options[num_nc_groups - 1].value;
			console.log('List of Nextcloud groups selected for profile ' + profile_data['cloudsuite_gears_profile_id'] + ': ' + nc_group_list);
		
			profile_data['cloudsuite_gears_nextcloud_groups'] = nc_group_list;
		} else {
			profile_data['cloudsuite_gears_nextcloud_groups'] = '';
		}

		return profile_data;
	}

	// Check if the initial signature profile setup routine has to be run in case 
	// the Migration step during the initial installation of the SignLive! app failed to do so
	$.ajax({
		url: OC.generateUrl('/apps/signlive/settings/check_initial_signature_profile_setup'),
		type: 'post',
		async : false,
		success: function(response) {
			if (response.success === true) {
				console.debug('Successfully checked for status of initial signature setup: ' + response.message);
			} else {
				console.error('Failed to check for status of initial signature profile setup! Error message: ' + response.message); 
			}
		},
		error: function(response) {
			console.error('Failed to check for status of initial signature profile setup! Error message: ' + response.message);
		}
	});

	toggleHideElement('json_placeholder_variables_help', true);

	queryAllNextcloudGroupsFromBackend();
	queryProfileSelectionListFromBackend();

	// Don't select a specific profile when the page is (re)loaded
	// Just fill the selection list in alphabetic order
	updateProfileSelectionList();
	var selected_profile = document.getElementById('profile_select').options.item(document.getElementById('profile_select').selectedIndex).value;
	if (selected_profile !== null && selected_profile !== undefined) {
		updateSignatureProfileForm(selected_profile);
	} else {
		console.warn('No valid signature profile definitions have been found in the Nextcloud settings! Can not update signature profile list.');
	}

	document.getElementById('profile_select').onchange = function() {
		var index = document.getElementById('profile_select').selectedIndex;
		if (index >= 0) {
			var selected_profile = document.getElementById('profile_select').options.item(index).value;
			console.debug('Selected profile: ' + selected_profile);
			
			clearProfileStatusMessage();
			
			updateSignatureProfileForm(selected_profile);
		}
	}

	document.getElementById('cloudsuite_gears_signature_add_groups').onclick = function () {
		console.log('Adding selected groups for current profile.');
		var groups_to_add = [];
		var ng_select = document.getElementById('cloudsuite_gears_signature_all_nextcloud_groups');
		for (var i = 0; i <  ng_select.options.length; i++) {
			var ng_group = ng_select.options[i];
			if (ng_group.selected) {
				console.debug('Selected group: ' + ng_group.text);
				groups_to_add.push(ng_group.text);
			}
		}
		var ng_profile_groups = document.getElementById('cloudsuite_gears_signature_profile_nextcloud_groups');
		console.log('Groups to add: ' + groups_to_add.length);
		for (var i = 0; i < groups_to_add.length; i++) {
			var add_group = true;
			for (var j = 0; j < ng_profile_groups.options.length; j++) {
				if (groups_to_add[i] === ng_profile_groups.options[j].text) {
					add_group = false;
				}
			}

			if (add_group === true && groups_to_add[i] !== '') {
				var ng_option = document.createElement('option');
				ng_option.value = groups_to_add[i];
				ng_option.text = groups_to_add[i];
				ng_profile_groups.options.add(ng_option);
			}
		}
	}

	document.getElementById('cloudsuite_gears_signature_remove_groups').onclick = function () {
		console.log('Removing selected groups from current profile.');
		var groups_to_remove = [];
		var ng_select = document.getElementById('cloudsuite_gears_signature_profile_nextcloud_groups');
		for (var i = 0; i <  ng_select.options.length; i++) {
			var ng_group = ng_select.options[i];
			if (ng_group.selected) {
				console.log('Removing group: ' + ng_group.text);
				groups_to_remove.push(ng_group.text);
			}
		}
		console.log('Groups to remove: ' + groups_to_remove.length);
		for (var i = 0; i < groups_to_remove.length; i++) {
			for (var j = 0; j < ng_select.options.length; j++) {
				if (groups_to_remove[i] === ng_select.options[j].text) {
					ng_select.options[j].remove();
				}
			}
		}
	}

	document.getElementById('cloudsuite_gears_add_signature_profile_button').onclick = function (event) {
		if (document.getElementById('profile_select').selectedIndex >= 0) {
			event.preventDefault();

			var selected_profile_id = document.getElementById('profile_select').options[document.getElementById('profile_select').selectedIndex].value;
			var selected_profile_name = document.getElementById('profile_select').options[document.getElementById('profile_select').selectedIndex].text;
			console.debug('Copy and save new signature profile: ' + selected_profile_id + ' named ' + selected_profile_name);

			// Filter out reserved values included in the profiles list: Unlikely a profile would be named like this, but check to be safe 
			if (selected_profile_name === 'menu_entry_priority_value' ||
				selected_profile_name === 'relative_app_path') {
				OC.msg.startSaving('#cloudsuite_gears_signature_profiles_notifications_msg', t('signlive', 'Checking for reserved profile names.'));
				console.error('Can not create new profile! Profile must not be named menu_entry_priority_value or relative_app_path.');
				OC.msg.finishedError('#cloudsuite_gears_signature_profiles_notifications_msg', t('signlive', 'Failed to save newly added signature profile') + ': ' + '\'' + selected_profile_name + '\'!' + t('signlive', 'Profiles must not be named menu_entry_priority_value or relative_app_path') + '!');
			
				return;
			}

			var new_profile_id = 'cloudsuite_gears_signature_profile_id_' + (highest_profile_id + 1) + ';;';
			var new_profile_id_len = new_profile_id.length;
			// We need to accomodate the length of configkeys in the oc_appsettings table.
			var new_profile_text_id_len = 64 - new_profile_id_len;
			if (new_profile_text_id_len <= 0) {
				OC.msg.startSaving('#cloudsuite_gears_signature_profiles_notifications_msg', t('signlive', 'Checking for valid profile ID length.'));
				console.error('Can not create new profile! Profile ID would not fit into the Nexcloud app settings database table. Please choose a shorter profile name.');
				OC.msg.finishedError('#cloudsuite_gears_signature_profiles_notifications_msg', t('signlive', 'Failed to save newly added signature profile') + ' \'' + profile_data['cloudsuite_gears_profile_name'] + '\'! ' + t('signlive', 'Profile ID would not fit into the Nexcloud app settings database table. Please choose a shorter profile name.'));
			
				return;
			} else {
				console.debug('A newly added signature profile needs to be stored in the Nextcloud settings.');

				var profile_data = getSelectedProfileData();
				var highest_profile_id = -1;
				for (profile_id in cloudsuite_gears_all_profiles) {
					var profile_unique_id = profile_id.split(';;')[0];
					profile_unique_id = profile_unique_id.replace('cloudsuite_gears_profile_id_', '');
					var profile_numeric_id = parseInt(profile_unique_id);
					if (highest_profile_id < profile_numeric_id) {
						console.debug('New highest numeric profile ID: ' + profile_numeric_id);
						highest_profile_id = profile_numeric_id;
					}
				}

				new_profile_id = (highest_profile_id + 1) + ';;' + profile_data['cloudsuite_gears_profile_name'];
				console.debug('Assigned profile ID for newly created signature profile: ' + new_profile_id);

				// Update profile data with new profile ID and profile display name with suffix
				// The user needs to change the profile display name explicitly later
				profile_data['cloudsuite_gears_profile_id'] = new_profile_id;
				profile_data['cloudsuite_gears_profile_name'] = profile_data['cloudsuite_gears_profile_name'] + ' Copy';
				
				OC.msg.startSaving('#cloudsuite_gears_signature_profiles_notifications_msg', t('signlive', 'Saving new signature profile') + ' ' + profile_data['cloudsuite_gears_profile_name']);
				$.ajax({
					url: OC.generateUrl('/apps/signlive/settings/add_signature_profile'),
					type: 'post',
					data: {
						'cloudsuite_gears_profile_id': new_profile_id,
						'cloudsuite_gears_profile_name': profile_data['cloudsuite_gears_profile_name'],
						'cloudsuite_gears_profile_type': profile_data['cloudsuite_gears_profile_type'],
						'cloudsuite_gears_profile_json': profile_data['cloudsuite_gears_profile_json'],
						'cloudsuite_gears_default_signature_profile': profile_data['cloudsuite_gears_default_signature_profile'],
						'cloudsuite_gears_nextcloud_groups': profile_data['cloudsuite_gears_nextcloud_groups'],
						'cloudsuite_gears_signature_profile_menu_priority': profile_data['cloudsuite_gears_signature_profile_menu_priority']
					},
					async : false,
					success: function(response) {
						if (response.success === true) {
							console.debug('Saved newly added signature profile \'' + profile_data['cloudsuite_gears_profile_name'] + '\' to Nextcloud settings: ' + response.message);

							OC.msg.finishedSuccess('#cloudsuite_gears_signature_profiles_notifications_msg', t('signlive', 'Saved newly added signature profile to Nextcloud settings') + ': ' + '\'' + profile_data['cloudsuite_gears_profile_name'] + '\'');
						} else {
							console.error('Failed to save newly added signature profile! Error message: ' + response.message); 
							console.error('Error details: ' + response.details);

							OC.msg.finishedError('#cloudsuite_gears_signature_profiles_notifications_msg', t('signlive', 'Failed to save newly added signature profile') + ': ' + '\'' + profile_data['cloudsuite_gears_profile_name'] + '\'!');
						}

						queryProfileSelectionListFromBackend();
						queryAllNextcloudGroupsFromBackend();

						var new_selected_profile_id = '';
						var new_selected_profile_name = '';
						if (response.success === false) {
							new_selected_profile_id = selected_profile_id;
							new_selected_profile_name = selected_profile_name;
						} else {
							new_selected_profile_id = new_profile_id;
							new_selected_profile_name = profile_data['cloudsuite_gears_profile_name'];
						}
						
						console.debug('New profile name to select: ' + new_selected_profile_name);

						sorted_profile_name_data = sortProfileList();
						var new_profile_name_index = getPositionOfNewProfileEntry(sorted_profile_name_data, new_selected_profile_name, selected_profile_name, true);
						updateProfileSelectionList(new_profile_name_index);

						updateSignatureProfileForm(new_selected_profile_id);
					},
					error: function(response) {
						console.error(t('signlive', 'Failed to save newly added signature profile to Nextcloud settings') + ': \'' + profile_data['cloudsuite_gears_profile_name'] + '\'!' + t('signlive', 'Error message') + ': ' + response.message);
						OC.msg.finishedError('#cloudsuite_gears_signature_profiles_notifications_msg', t('signlive', 'Failed to save newly added signature profile to Nextcloud settings') + ': \'' + profile_data['cloudsuite_gears_profile_name'] + '\'!' + t('signlive', 'Error message') + ': ' + response.message);
						queryProfileSelectionListFromBackend();
						queryAllNextcloudGroupsFromBackend();
						
						updateProfileSelectionList();
						updateSignatureProfileForm(selected_profile_id);
					}
				});
			}
		} else {
			console.warn('Can not save copy of selected profile: No profile to copy from is selected!');
			OC.msg.finishedError('#cloudsuite_gears_signature_profiles_notifications_msg', t('signlive', 'Can not save copy of selected profile: No profile to copy from is selected!'));
		}
	}

	document.getElementById('cloudsuite_gears_edit_signature_profile_button').onclick = function (event) {
		console.debug('Selected profile index in selection control: ' + document.getElementById('profile_select').selectedIndex);
		if (document.getElementById('profile_select').selectedIndex >= 0) {
			var selected_profile = document.getElementById('profile_select').options[document.getElementById('profile_select').selectedIndex].text;
			console.debug('Saving modified signature profile: ' + selected_profile);
			event.preventDefault();

			profile_data = getSelectedProfileData();

			console.debug('An already existing profile needs to be updated in the Nextcloud settings: ' + selected_profile);
			var currently_selected_profile = profile_data['cloudsuite_gears_profile_id'];
			console.debug('Profile ID to update: ' + currently_selected_profile);

			var currently_selected_profile_name = profile_data['cloudsuite_gears_profile_name'];
			// Filter out reserved values included in the profiles list: Unlikely a profile would be named like this, but check to be safe 
			if (currently_selected_profile_name === 'menu_entry_priority_value' ||
				currently_selected_profile_name === 'relative_app_path') {
				OC.msg.startSaving('#cloudsuite_gears_signature_profiles_notifications_msg', t('signlive', 'Checking for reserved profile names.'));
				console.error('Can not save existing profile! Profile must not be named menu_entry_priority_value or relative_app_path.');
				OC.msg.finishedError('#cloudsuite_gears_signature_profiles_notifications_msg', t('signlive', 'Failed to save signature profile') + ': ' + '\'' + selected_profile + '\'!' + t('signlive', 'Profiles must not be named menu_entry_priority_value or relative_app_path') + '!');
			
				return;
			}

			// Check if the provided JSON section is valid
			var json_section_valid = false;
			OC.msg.startSaving('#cloudsuite_gears_json_validation_msg', t('signlive', 'Validating provided JSON section.'));
			$.ajax({
				url: OC.generateUrl('/apps/signlive/settings/validate_json_section'),
				type: 'post',
				data: {
					'cloudsuite_gears_profile_id': currently_selected_profile,
					'cloudsuite_gears_profile_name': profile_data['cloudsuite_gears_profile_name'],
					'cloudsuite_gears_profile_json': profile_data['cloudsuite_gears_profile_json'],
				},
				async : false,
				success: function(response) {
					if (response.success) {
						json_section_valid = true;
						console.debug('JSON section validated OK: ' + response.message);
						OC.msg.finishedSuccess('#cloudsuite_gears_json_validation_msg', t('signlive', 'JSON section is valid, please proceed to save your newly created signature profile.'));
					} else {
						console.debug('JSON section validated as invalid: ' + response.message);
						console.warn('Provided JSON section contains invalid variable substitution placeholders, will not save new profile before detected errors are corrected.');
						OC.msg.finishedError('#cloudsuite_gears_json_validation_msg', response.message);
					}
				},
				error: function(response) {
					console.error('Could not validate JSON section before saving new profile \'' + currently_selected_profile + '\': Backend service for JSON syntax validation could not be reached!');
					OC.msg.finishedError('#cloudsuite_gears_json_validation_msg', 'Could not validate JSON section before saving new profile \'' + currently_selected_profile + '\': Backend service for JSON syntax validation could not be reached!');
				}
			});

			console.debug('JSON section valid: ' + json_section_valid);
			if (json_section_valid === true) {
				OC.msg.startSaving('#cloudsuite_gears_signature_profiles_notifications_msg', t('signlive', 'Saving changes'));
				$.ajax({
					url: OC.generateUrl('/apps/signlive/settings/edit_signature_profile'),
					type: 'post',
					data: {
						'cloudsuite_gears_profile_id': profile_data['cloudsuite_gears_profile_id'],
						'cloudsuite_gears_profile_name': profile_data['cloudsuite_gears_profile_name'],
						'cloudsuite_gears_profile_type': profile_data['cloudsuite_gears_profile_type'],
						'cloudsuite_gears_profile_json': profile_data['cloudsuite_gears_profile_json'],
						'cloudsuite_gears_default_signature_profile': profile_data['cloudsuite_gears_default_signature_profile'],
						'cloudsuite_gears_nextcloud_groups': profile_data['cloudsuite_gears_nextcloud_groups'],
						'cloudsuite_gears_signature_profile_menu_priority': profile_data['cloudsuite_gears_signature_profile_menu_priority']
					},
					async : false,
					success: function(response) {
						console.debug('Saved modified signature profile \'' + selected_profile + '\' to Nextcloud settings: ' + response.message);

						OC.msg.finishedSuccess('#cloudsuite_gears_signature_profiles_notifications_msg', 'Successfully saved modified signature profile \'' + selected_profile + '\' to Nextcloud settings.');

						queryProfileSelectionListFromBackend();
						queryAllNextcloudGroupsFromBackend();

						var profile_name_in_form = document.getElementById('cloudsuite_gears_user_profile_name').value;
						var sorted_profile_data = sortProfileList();
						var new_profile_name_index = getPositionOfNewProfileEntry(sorted_profile_data, profile_name_in_form, profile_data['cloudsuite_gears_profile_name'], false);

						updateProfileSelectionList(new_profile_name_index);

						var profile_name_for_form_update = sorted_profile_data['profile_names_and_ids'][profile_data['cloudsuite_gears_profile_name']];
						updateSignatureProfileForm(profile_name_for_form_update);
					},
					error: function(response) {
						console.error('Failed to save modified signature profile \'' + selected_profile + '\' to Nextcloud settings! Error message: ' + response.message);
						OC.msg.finishedError('#cloudsuite_gears_signature_profiles_notifications_msg', 'Failed to save modified signature profile \'' + selected_profile + '\' to Nextcloud settings!');
					}
				});
			} else {
				OC.msg.finishedError('#cloudsuite_gears_signature_profiles_notifications_msg', 'Did not save modified signature profile \'' + selected_profile + '\' to Nextcloud settings: Provided JSON section is invalid, please check for syntax errors or invalid variable placeholders.');
			}
		} else {
			console.warn('Can not commit changes for edited profile: No profile is selected!');
		}
	}

	document.getElementById('cloudsuite_gears_delete_signature_profile_button').onclick = function (event) {
		if (document.getElementById('profile_select').selectedIndex >= 0) {
			var selected_profile = document.getElementById('profile_select').options[document.getElementById('profile_select').selectedIndex].text;
			var selected_profile_id = document.getElementById('profile_select').options[document.getElementById('profile_select').selectedIndex].value;
			console.debug('Deleting signature profile: ' + selected_profile + ' (profile ID: ' + selected_profile_id + ')');
			event.preventDefault();

			OC.msg.startSaving('#cloudsuite_gears_signature_profiles_notifications_msg', t('signlive', 'Deleting signature profile...'));
			var confirmation = confirm(t('signlive', 'Do you really want to delete the signature profile') + '\'' + selected_profile + '\'?' + t('signlive', 'This action can not be undone.'));
			if (confirmation === true) {
				console.debug('Delete operation confirmed, proceeding.');
				$.ajax({
					url: OC.generateUrl('/apps/signlive/settings/delete_signature_profile'),
					type: 'post',
					data: {
						cloudsuite_gears_profile_id: selected_profile_id
					},
					async : false,
					success: function(response) {
						console.debug('Successfully deleted signature profile \'' + selected_profile + '\' from Nextcloud settings: ' + response.message);

						queryProfileSelectionListFromBackend();
						if (cloudsuite_gears_all_profiles !== null && cloudsuite_gears_all_profiles !== undefined &&
							cloudsuite_gears_signature_profile_count > 0) {

						 	var sorted_profile_data = sortProfileList();
							// Select first entry in signature profile selection list after deleting a profile
							updateProfileSelectionList(0);
							if (document.getElementById('profile_select').options.length > 0) {
								document.getElementById('profile_select').options.selectedIndex = 0;

								var new_selected_profile = sorted_profile_data['sorted_profile_names'][0];
								console.debug('Selecting new profile after deletion: ' + new_selected_profile);
								var new_selected_profile_id = sorted_profile_data['profile_names_and_ids'][new_selected_profile];
								updateSignatureProfileForm(new_selected_profile_id);
								OC.msg.finishedSuccess('#cloudsuite_gears_signature_profiles_notifications_msg', 'Successfully deleted signature profile \'' + selected_profile + '\' from Nextcloud settings.');
							}
						} else {
							console.error('Received no or an empty list of signature profiles from the Nextcloud backend after profile deletion!');
							OC.msg.finishedError('#cloudsuite_gears_signature_profiles_notifications_msg', 'Received no or an empty list of signature profiles from the Nextcloud backend after deleting profile \'' + selected_profile + '\' from Nextcloud settings!');
						}
					},
					error: function(response) {
						console.error('Failed to delete signature profile \'' + selected_profile + '\' from Nextcloud settings! Error message: ' + response.message);
						OC.msg.finishedError('#cloudsuite_gears_signature_profiles_notifications_msg', 'Failed to delete signature profile \'' + selected_profile + '\' from Nextcloud settings!');
					}
				});
			} else {
				console.log('Delete operation canceled, doing nothing.');
				OC.msg.finishedSuccess('#cloudsuite_gears_signature_profiles_notifications_msg', t('signlive', 'Delete operation canceled, doing nothing.'));
			}
		} else {
			console.warn('Can not delete profile: No profile to delete is selected!');
			OC.msg.finishedError('#cloudsuite_gears_signature_profiles_notifications_msg', 'Can not delete profile, no profile to delete is selected!');
		}
	}

	document.getElementById('cloudsuite_gears_signature_profile_factory_reset').onclick = function (event) {
		console.debug('Reset of signature profiles to factory defaults requested.');

		event.preventDefault();

		var confirmation = confirm(t('signlive', 'Do you really want to delete all customized signature profiles and re-initialize SignLive! with the default signature profile definitions') + '?' + t('signlive', 'This action can not be undone.'));
		if (confirmation === true) {
			console.debug('Signature profile reset operation confirmed, proceeding.');

			// First remove the flag indicating that the initial signature profile generation has already run
			var initial_profile_creation_flag_removed = false;
			$.ajax({
				url: OC.generateUrl('/apps/signlive/settings/reset_profile_creation_flag'),
				type: 'post',
				async : false,
				success: function(response) {
					console.debug('Successfully removed initial signature profile creation flag from Nextcloud settings: ' + response.message);
					initial_profile_creation_flag_removed = true;
				},
				error: function(response) {
					console.error('Failed to remove initial signature profile creation flag from Nextcloud settings: ' + response.message + '!');
				}
			});
			
			// Then delete all existing signature profile definitions and recreate the initial factory default set of signature profiles
			var signature_profiles_reinitialized = false;
			if (initial_profile_creation_flag_removed === true) {
				$.ajax({
					url: OC.generateUrl('/apps/signlive/settings/profiles_factory_reset'),
					type: 'post',
					async : false,
					success: function(response) {
						signature_profiles_reinitialized = true;
						console.debug('Successfully re-initialized SignLive! with factory default signature profiles: ' + response.message);
					},
					error: function(response) {
						console.error('Failed to re-initialize SignLive! with factory default signature profiles: ' + response.message + '!');
					}
				});
			}

			// Only update signature profile data form controls if the re-initialization operation was successful
			if (signature_profiles_reinitialized === true) {
				queryProfileSelectionListFromBackend();
				queryAllNextcloudGroupsFromBackend();

				// Default to first signature profile in vanilla list
				updateProfileSelectionList(0);
				var sorted_profile_data = sortProfileList();
				var new_selected_profile_name = sorted_profile_data['sorted_profile_names'][0];
				var new_selected_profile_id = sorted_profile_data['profile_names_and_ids'][new_selected_profile_name];
				updateSignatureProfileForm(new_selected_profile_id);
			}
		} else {
			console.debug('Signature profile reset operation cancelled.');
		}
	}

	// Manually add saveSettings() handlers for all settings inputs not related to individual signature profiles
	document.getElementById('cloudsuite_gears_http_protocol_type').addEventListener('change', (event) => {
		saveSettings();
	});
	document.getElementById('cloudsuite_gears_host').addEventListener('change', (event) => {
		saveSettings();
	});
	document.getElementById('cloudsuite_gears_port').addEventListener('change', (event) => {
		saveSettings();
	});
	document.getElementById('cloudsuite_gears_gears_core_path').addEventListener('change', (event) => {
		saveSettings();
	});

	document.getElementById('cloudsuite_gears_show_default_signature_entries_in_context_menu').addEventListener('change', (event) => {
		saveSettings();
	});
	document.getElementById('cloudsuite_gears_display_menu_priorities').addEventListener('change', (event) => {
		saveSettings();
	});

	document.getElementById('cloudsuite_gears_global_attribute_company_name').addEventListener('change', (event) => {
		saveSettings();
	});

	document.getElementById('cloudsuite_gears_notification_duration').addEventListener('change', (event) => {
		saveSettings();
	});
	document.getElementById('cloudsuite_gears_notification_check_interval').addEventListener('change', (event) => {
		saveSettings();
	});
	document.getElementById('cloudsuite_gears_show_success_notifications').addEventListener('change', (event) => {
		saveSettings();
	});
	document.getElementById('cloudsuite_gears_show_verbose_failure_notifications').addEventListener('change', (event) => {
		saveSettings();
	});

	document.getElementById('cloudsuite_gears_json_section_help').onclick = function (event) {
		console.debug('Show/hide JSON section help text -- current style: ' + document.getElementById('json_placeholder_variables_help').style.display);
		
		if (document.getElementById('json_placeholder_variables_help').style.display === "block") {
			console.debug('HIDE JSON section help text');
			toggleHideElement('json_placeholder_variables_help', true);
		}
		else if (document.getElementById('json_placeholder_variables_help').style.display === "none" || 
				 document.getElementById('json_placeholder_variables_help').style.display === "") {
			console.debug('SHOW JSON section help text');
			toggleHideElement('json_placeholder_variables_help', false);
		}
	}

	document.getElementById('cloudsuite_gears_signature_profile_json_label').onclick = function (event) {
		console.debug('Show/hide JSON section help text -- current style: ' + document.getElementById('json_placeholder_variables_help').style.display);
		
		if (document.getElementById('json_placeholder_variables_help').style.display === "block") {
			console.debug('HIDE JSON section help text');
			toggleHideElement('json_placeholder_variables_help', true);
		}
		else if (document.getElementById('json_placeholder_variables_help').style.display === "none" || 
			document.getElementById('json_placeholder_variables_help').style.display === "") {
			console.debug('SHOW JSON section help text');
			toggleHideElement('json_placeholder_variables_help', false);
		}
	}

	document.getElementById('cloudsuite_gears_signature_default_profile').addEventListener('change', (event) => {
		console.debug('Default profile option activated previously: ' + default_profile_option_value);
		default_profile_option_value = !default_profile_option_value;
		console.debug('Default profile option after flipping: ' + default_profile_option_value);
		if (default_profile_option_value === true) {
			console.debug('DISABLING (disabled = true, enabled_value = false) Nextcloud group selection controls');
			toggleEnableElement('cloudsuite_gears_signature_all_nextcloud_groups', false);
			toggleEnableElement('cloudsuite_gears_signature_profile_nextcloud_groups', false);
			toggleEnableElement('cloudsuite_gears_signature_add_groups', false);
			toggleEnableElement('cloudsuite_gears_signature_remove_groups', false);
			toggleEnableElement('cloudsuite_gears_signature_nextcloud_groups_label', false);
		}
		else if (default_profile_option_value === false) {
			console.debug('ENABLING (disabled = false, enabled_value = true) Nextcloud group selection controls');
			toggleEnableElement('cloudsuite_gears_signature_all_nextcloud_groups', true);
			toggleEnableElement('cloudsuite_gears_signature_profile_nextcloud_groups', true);
			toggleEnableElement('cloudsuite_gears_signature_add_groups', true);
			toggleEnableElement('cloudsuite_gears_signature_remove_groups', true);
			toggleEnableElement('cloudsuite_gears_signature_nextcloud_groups_label', true);
		}
	});
});
