$(document).ready(function () {
    if (OCA == null || OCA.Files == null || OCA.Files.fileActions == null) {
        console.warn('OCA.Files.fileActions instance is invalid or not yet defined, can not register custom file action!');
        return;
    }

    // See: https://stackoverflow.com/questions/105034/how-to-create-a-guid-uuid#answer-2117523
    function generateUUID() {
        var d = new Date().getTime();//Timestamp
        var d2 = ((typeof performance !== 'undefined') && performance.now && (performance.now()*1000)) || 0;//Time in microseconds since page-load or 0 if unsupported
        return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
            var r = Math.random() * 16;//random number between 0 and 16
            if(d > 0){ //Use timestamp until depleted
                r = (d + r)%16 | 0;
                d = Math.floor(d/16);
            } else { //Use microseconds since page-load if supported
                r = (d2 + r)%16 | 0;
                d2 = Math.floor(d2/16);
            }
            return (c === 'x' ? r : (r & 0x3 | 0x8)).toString(16);
        });
    }

    var public_callback_params = [];
    var public_callback_functions = [];
    var public_profile_names = [];
    var public_profile_types = [];
    var public_profile_identifiers = [];

    var restricted_callback_params = [];
    var restricted_callback_functions = [];
    var restricted_profile_names = [];
    var restricted_profile_types = [];
    var restricted_profile_identifiers = [];

    function registerMenuItems(process_restricted_profiles, profile_unique_ids, profile_names_and_types_to_use, profile_indices_and_menu_labels_to_use, profile_indices_and_signature_types_to_use, num_profiles_to_use, show_menu_priority_values, profile_menu_priorities) {
        for (var i = 0; i < num_profiles_to_use; i++) {
            var profile_unique_id = profile_unique_ids[i];
            var profile_name = profile_indices_and_menu_labels_to_use[i];
            var profile_type = profile_names_and_types_to_use[i];
            var signature_type = profile_indices_and_signature_types_to_use[i];
            var profile_identifier = i + "::" + profile_name + "::" + profile_type;

            if (process_restricted_profiles == true) {
                restricted_profile_identifiers[i] = profile_unique_ids[i];
                restricted_profile_names[i] = profile_name;
                restricted_profile_types[i] = profile_type;
                restricted_callback_params[i] = {profile_unique_id: profile_unique_id, profile_identifier: profile_identifier, profile_index: i, profile_type: profile_type, profile_name: profile_name, signature_type: signature_type};
            } else {
                public_profile_identifiers[i] = profile_unique_ids[i];
                public_profile_names[i] = profile_name;
                public_profile_types[i] = profile_type;
                public_callback_params[i] = {profile_unique_id: profile_unique_id, profile_identifier: profile_identifier, profile_index: i, profile_type: profile_type, profile_name: profile_name, signature_type: signature_type};
            }

            var callbackFunctionForSignature = function (filename, context) {
                var dir = context.dir || context.fileList.getCurrentDirectory();
                var original_folder_uri = window.location.protocol + '//' + window.location.host + '/index.php/apps/files/?dir=' + dir;

                console.debug("Profile unique ID: " + this.profile_unique_id);
                console.debug("Signing document '" + filename + "' using profile index " + this.profile_index + ": " + this.profile_name);
                console.debug("Profile type: " + this.profile_type);
                console.debug("Profile identifer: " + this.profile_identifier);
                console.debug("Signature type: " + this.signature_type);

                console.debug("Current directory as reported by file UI context: " + dir); 
		        var file_path = '';
                if (dir != '') {
                    file_path =  dir + '/' + filename;
                } else {
                    file_path = '/' + filename;
                }

                if (file_path.startsWith('//')) {
                    file_path = file_path.substr(1);
                    console.debug('Removing double \'//\' from file_path: ' + file_path);
                }

                console.debug("Nextcloud-side path for file to sign: " + file_path); 

                var rest_url = '';
            
                if (this.signature_type == 'viewer_based_signature') {
                    rest_url = OC.generateUrl('/apps/signlive/sign_document/sign_document_with_viewer_job');
                } else if (this.signature_type == 'automatic_signature') {
                    rest_url = OC.generateUrl('/apps/signlive/sign_document/sign_document_with_profile_job');
                }
            
                var download_url = context.fileList.getDownloadUrl(filename, dir, false);
                console.debug('Download URL for file ' + filename + ': ' + download_url);

                // URL used for first iteration of backend signature request controller  
                // var redirect_uri = window.location.protocol + '//' + window.location.host + '/index.php/apps/signlive/sign_document/viewer_result?document_file_name=' + dir + "/" + filename + "&original_folder_uri=" + original_folder_uri;
                // URL for second (job-based) iteration of backend signature request controller
                // Encodes:
                // * Document file name
                // * Original folder of the unsigned document in the Nextcloud storage space
                // * UUID identifier - signature job UUID
                var signature_request_uuid = generateUUID(); 

                var redirect_uri = redirect_uri = window.location.protocol + '//' + window.location.host + '/index.php/apps/signlive';
                redirect_uri += '/sign_document/process_viewer_job_response?job_uuid=' + signature_request_uuid; // 'document_file_name=' + dir + "/" + filename + "&original_folder_uri=" + original_folder_uri;

                console.debug('=================================================================');
                console.debug('Calling REST backend service: ' + rest_url);
                console.debug('Using redirect URL for Gears response processing: ' + redirect_uri);
                console.debug('=================================================================');

                if (rest_url != '') {
                    console.debug('Creating signature request using profile ID: ' + this.profile_unique_id);
                    $.ajax({
                        url: rest_url,
                        type: 'post',
                        data: {
                            filename: file_path,
                            redirect_uri: redirect_uri,
                            profile_unique_id: this.profile_unique_id,
                            profile_name: this.profile_name,
                            profile_type: this.profile_type,
                            profile_index: this.profile_index,
                            signature_type: this.signature_type,
                            profile_identifier: this.profile_identifier,
                            job_uuid: signature_request_uuid,
                            download_url: download_url,
                            original_folder_uri: original_folder_uri
                        },
                        success: function(response) {
                            console.debug("Response received from Gears backend service: " + response.message);
                            if (typeof response.signature_type !== 'undefined') {
                                console.debug('signature_type for profile: \'' + response.signature_type + '\'')
                                if (response.signature_type == 'viewer_based_signature') {
                                    if (response.viewer_url !== '') {
                                        console.debug('Opening signature viewer in new browser tab: ' + response.viewer_url);
                                        window.open(response.viewer_url, '_self', '', false);
                                    } else {
                                        console.error('Received a response for a viewer-based signature request, but the URL returned by the Gears backend was empty!');
                                    }
                                }  else if (response.signature_type == 'automatic_signature') {
                                    console.debug('Received a response for an automatic signature request.');
                                }
                            }
                        },
                        error: function(response) {
                            console.error("Failed to create a signing request for file: " + filename + " using signature profile type " + profile_names_and_types_to_use[i]);
                        }
                    });
                }
            }

            if (process_restricted_profiles) {
                restricted_callback_functions[i] = callbackFunctionForSignature;
            } else {
                public_callback_functions[i] = callbackFunctionForSignature;
            }
        }
    }

    var public_signature_profiles = null;
    var num_public_signature_profiles = 0;
    var restricted_signature_profiles = null;
    var num_restricted_signature_profiles = 0;
    var show_public_profiles_in_context_menu = false;
    var show_menu_priority_values = false;
    var relative_app_path = '';

    $.ajax({
        url: OC.generateUrl('/apps/signlive/sign_document/get_signature_profiles_for_current_user'),
        type: 'post',
        async : false,
        success: function(response) {
            num_public_signature_profiles = parseInt(response.data.num_public_profiles);
            public_signature_profiles = response.data.public_signature_profiles;
            show_public_profiles_in_context_menu = response.data.show_public_profiles_in_context_menu;
            num_restricted_signature_profiles = parseInt(response.data.num_restricted_profiles);
            restricted_signature_profiles = response.data.restricted_signature_profiles;
            show_menu_priority_values = response.data.cloudsuite_gears_display_menu_priorities;
            relative_app_path = response.data.relative_app_path;

            console.debug('Response received from /sign_document/get_signature_profiles_for_current_user handler: ' + num_restricted_signature_profiles + ' restricted signature profiles visible for current user.');
        },
        error: function(response) {
            console.error("Failed to retrieve list of signature profiles for current user! Will only show public signature profiles in context menu (if any).");
        }
    });

    if (public_signature_profiles != null || restricted_signature_profiles != null) {
        console.debug('Got valid entries for either public or restricted signature profiles. Public profiles: ' + num_public_signature_profiles + '; restricted profiles: ' + num_restricted_signature_profiles);
        
        console.debug('Restricted profile data: ' + JSON.stringify(restricted_signature_profiles));
        console.debug('Public     profile data: ' + JSON.stringify(public_signature_profiles));
        console.debug('num_restricted_signature_profiles = ' + num_restricted_signature_profiles);
        console.debug('num_public_signature_profiles     = ' + num_public_signature_profiles);

        var public_profile_indices_and_types_to_use = [];
        var public_profile_unique_ids = [];
        var public_profile_indices_and_menu_labels_to_use = [];
        var public_profile_indices_and_signature_types_to_use = [];

        var restricted_profile_indices_and_types_to_use = [];
        var restricted_profile_unique_ids = [];
        var restricted_profile_indices_and_menu_labels_to_use = [];
        var restricted_profile_indices_and_signature_types_to_use = [];

        var public_profile_priority_values = {};
        var restricted_profile_priority_values = {};

        var profile_priority_values = {};

        if (public_signature_profiles !== null && num_public_signature_profiles > 0) {
            var i = 0;
            for (const profile_id in public_signature_profiles) {
                public_profile_indices_and_types_to_use[i] = 'public_profile';
                public_profile_unique_ids[i] = profile_id;

                profile_priority_values[profile_id] = public_signature_profiles[profile_id]['profile_menu_priority'];
                if (show_menu_priority_values === true ||
                    show_menu_priority_values === "true") {
                    public_profile_indices_and_menu_labels_to_use[i] = public_signature_profiles[profile_id]['profile_name'] + ' (priority: ' + profile_priority_values[profile_id] + ')';
                } else {
                    public_profile_indices_and_menu_labels_to_use[i] = public_signature_profiles[profile_id]['profile_name'];
                }

                console.debug('Adding public profile label: ' + public_profile_indices_and_menu_labels_to_use[i]);
                public_profile_indices_and_signature_types_to_use[i] = public_signature_profiles[profile_id]['profile_type'];
                console.debug('Adding public profile type: ' + public_profile_indices_and_signature_types_to_use[i]);
                public_profile_priority_values[profile_id] = public_signature_profiles[profile_id]['profile_menu_priority'];


                console.debug('Adding public profile menu priority: ' + public_profile_priority_values[profile_id]);
                i = i + 1;
            }
        }

        if (restricted_signature_profiles !== null && num_restricted_signature_profiles > 0) {            
            var i = 0;
            for (const profile_id in restricted_signature_profiles) {
                console.debug('Restricted profile ID: ' + profile_id);
                restricted_profile_indices_and_types_to_use[i] = 'restricted_profile';
                restricted_profile_unique_ids[i] = profile_id;
                
                profile_priority_values[profile_id] = restricted_signature_profiles[profile_id]['profile_menu_priority'];
                if (show_menu_priority_values === true ||
                    show_menu_priority_values === "true") {
                    restricted_profile_indices_and_menu_labels_to_use[i] = restricted_signature_profiles[profile_id]['profile_name'] + ' (priority: ' + profile_priority_values[profile_id] + ')';
                } else {
                    restricted_profile_indices_and_menu_labels_to_use[i] = restricted_signature_profiles[profile_id]['profile_name'];
                }

                console.debug('Adding restricted profile label: ' + restricted_profile_indices_and_menu_labels_to_use[i]);
                restricted_profile_indices_and_signature_types_to_use[i] = restricted_signature_profiles[profile_id]['profile_type'];
                console.debug('Adding restricted profile type: ' + restricted_profile_indices_and_signature_types_to_use[i]);
                restricted_profile_priority_values[profile_id] = restricted_signature_profiles[profile_id]['profile_menu_priority'];
                
                console.debug('Adding restricted profile menu priority: ' + restricted_profile_priority_values[profile_id]);
                i = i + 1;
            }
        }

        var restricted_profile_priority_groups = [];
        var public_profile_priority_groups = [];
        console.debug('Restricted profile menu priorities: ');
        for (var profile_id in restricted_profile_priority_values) {
            console.debug('Profile ' + profile_id + ': ' + restricted_profile_priority_values[profile_id]);
            if (!(restricted_profile_priority_values[profile_id] in restricted_profile_priority_groups)) {
                console.debug('Registering new priority group: ' + restricted_profile_priority_values[profile_id]);
                restricted_profile_priority_groups[restricted_profile_priority_values[profile_id]] = [];
            }
            restricted_profile_priority_groups[restricted_profile_priority_values[profile_id]].push(profile_id);
        }
        console.debug('Public profile menu priorities: ');
        for (var profile_id in  public_profile_priority_values) {
            console.debug('Profile ' + profile_id + ': ' + public_profile_priority_values[profile_id]);
            if (!(public_profile_priority_values[profile_id] in public_profile_priority_groups)) {
                console.debug('Registering new priority group: ' + public_profile_priority_values[profile_id]);
                public_profile_priority_groups[public_profile_priority_values[profile_id]] = [];
            }
            public_profile_priority_groups[public_profile_priority_values[profile_id]].push(profile_id);
        }

        var restricted_profile_priority_tiers = [];
        var public_profile_priority_tiers = [];
        for (var priority in restricted_profile_priority_groups) {
            restricted_profile_priority_tiers.push(priority);
        }
        for (var priority in public_profile_priority_groups) {
            public_profile_priority_tiers.push(priority);
        }

        var sorted_restricted_profile_priority_tiers = restricted_profile_priority_tiers.sort().reverse();
        var sorted_public_profile_priority_tiers = public_profile_priority_tiers.sort().reverse();

        console.debug('Relative app path: ' + relative_app_path);
        var img_base_path = relative_app_path + 'signlive/img';

        console.debug('Calling registerMenuItems() for restricted signature profiles: ' + num_restricted_signature_profiles + ' restricted profiles to add.');
        registerMenuItems(true, restricted_profile_unique_ids, restricted_profile_indices_and_types_to_use, restricted_profile_indices_and_menu_labels_to_use, 
            restricted_profile_indices_and_signature_types_to_use, num_restricted_signature_profiles);

        for (const priority_group in sorted_restricted_profile_priority_tiers) {
            console.debug('Registering restricted signature profiles in priority group: ' + sorted_restricted_profile_priority_tiers[priority_group]);
            for (const profile_id_1 in restricted_profile_priority_values) {
                console.debug('Profile ' + profile_id_1 + ' priority group: ' + restricted_profile_priority_values[profile_id_1]);
                if (restricted_profile_priority_values[profile_id_1] === sorted_restricted_profile_priority_tiers[priority_group]) {
                    console.debug('Looking up index in unsorted lists for profile_id_1: ' + profile_id_1);
                    var profile_numeric_id = profile_id_1.split(';;')[0];
                    var profile_display_text = profile_id_1.split(';;')[1];
                    var profile_and_priority_matched = false;
                    var profile_index_in_unsorted_lists = -1;
                    var k = 0;
                    for (const profile_id_2 in restricted_profile_indices_and_menu_labels_to_use) {
                        var profile_numeric_id_2 = restricted_profile_unique_ids[profile_id_2].split(';;')[0];
                        console.debug('Comparing: ' + profile_display_text + ' === ' + restricted_profile_indices_and_menu_labels_to_use[profile_id_2] + ';' +profile_numeric_id + ' === ' + profile_numeric_id_2);
                        if (profile_numeric_id === profile_numeric_id_2) {
                            console.debug('Found match: ' + profile_id_1 + ' with index ' + k);
                            profile_and_priority_matched = true;
                            profile_index_in_unsorted_lists = k;
                            break;
                        }
                        k += 1;
                    }

                    if (profile_and_priority_matched === true) {
                        console.debug('========================================');
                        console.debug('Verify profile data for profile_id_1: ' + profile_id_1);
                        console.debug('restricted_profile_indices_and_menu_labels_to_use[' + profile_index_in_unsorted_lists + ']: ' + restricted_profile_indices_and_menu_labels_to_use[profile_index_in_unsorted_lists]);
                        console.debug('restricted_profile_indices_and_types_to_use[' + profile_index_in_unsorted_lists + ']: ' + restricted_profile_indices_and_types_to_use[profile_index_in_unsorted_lists]);
                        console.debug('========================================');
                    
                        if (restricted_profile_unique_ids[profile_index_in_unsorted_lists] !== null && restricted_profile_unique_ids[profile_index_in_unsorted_lists] !== undefined &&
                            restricted_profile_indices_and_menu_labels_to_use[profile_index_in_unsorted_lists] !== null && restricted_profile_indices_and_menu_labels_to_use[profile_index_in_unsorted_lists] !== undefined) {
                            console.debug('Adding restricted profile: ' + restricted_profile_indices_and_menu_labels_to_use[profile_index_in_unsorted_lists]);
                            
                            var icon_url = window.location.protocol + "//" + window.location.host + img_base_path + '/app_dark.svg';
                            OCA.Files.fileActions.registerAction({
                                name: restricted_profile_unique_ids[profile_index_in_unsorted_lists],
                                displayName: restricted_profile_indices_and_menu_labels_to_use[profile_index_in_unsorted_lists],
                                mime: 'application/pdf',
                                order: -restricted_profile_priority_values[profile_id_1],
                                permissions: OC.PERMISSION_UPDATE,
                                icon: icon_url,
                                actionHandler: restricted_callback_functions[profile_index_in_unsorted_lists].bind(restricted_callback_params[profile_index_in_unsorted_lists])
                            });
                        }
                    }
                } else {
                    console.debug('Skipping profile_id_1 ' + profile_id_1 + ', not in priority group ' + sorted_restricted_profile_priority_tiers[priority_group]);
                }
            }
        }

        console.debug('show_public_profiles_in_context_menu = ' + show_public_profiles_in_context_menu + ' (typeof = ' + typeof(show_public_profiles_in_context_menu) + ')');
        if (show_public_profiles_in_context_menu === true || show_public_profiles_in_context_menu === "true" || show_public_profiles_in_context_menu === 1) {
            console.debug('Calling registerMenuItems() for public signature profiles: ' + num_public_signature_profiles + ' public profiles to add.');
            registerMenuItems(false, public_profile_unique_ids, public_profile_indices_and_types_to_use, public_profile_indices_and_menu_labels_to_use, 
                public_profile_indices_and_signature_types_to_use, num_public_signature_profiles);
        }

        if (show_public_profiles_in_context_menu == true || show_public_profiles_in_context_menu == "true") {
            for (const priority_group in sorted_public_profile_priority_tiers) {
                console.debug('Registering public signature profiles in priority group: ' + sorted_public_profile_priority_tiers[priority_group]);
                for (const profile_id_1 in public_profile_priority_values) {
                    console.debug('Profile ' + profile_id_1 + ' priority group: ' + public_profile_priority_values[profile_id_1]);
                    if (public_profile_priority_values[profile_id_1] === sorted_public_profile_priority_tiers[priority_group]) {
                        console.debug('Looking up index in unsorted lists for profile_id_1: ' + profile_id_1);
                        var profile_numeric_id = profile_id_1.split(';;')[0];
                        var profile_display_text = profile_id_1.split(';;')[1];
                        var profile_and_priority_matched = false;
                        var profile_index_in_unsorted_lists = -1;
                        var k = 0;
                        for (const profile_id_2 in public_profile_indices_and_menu_labels_to_use) {
                            var profile_numeric_id_2 = public_profile_unique_ids[profile_id_2].split(';;')[0];
                            console.debug('Comparing: ' + profile_display_text + ' === ' + public_profile_indices_and_menu_labels_to_use[profile_id_2] + ';' +profile_numeric_id + ' === ' + profile_numeric_id_2);
                            if (profile_numeric_id === profile_numeric_id_2) {
                                console.debug('Found match: ' + profile_id_1 + ' with index ' + k);
                                profile_and_priority_matched = true;
                                profile_index_in_unsorted_lists = k;
                                break;
                            }
                            k += 1;
                        }
    
                        if (profile_and_priority_matched === true) {
                            console.debug('========================================');
                            console.debug('Verify profile data for profile_id_1: ' + profile_id_1);
                            console.debug('public_profile_indices_and_menu_labels_to_use[' + profile_index_in_unsorted_lists + ']: ' + public_profile_indices_and_menu_labels_to_use[profile_index_in_unsorted_lists]);
                            console.debug('public_profile_indices_and_types_to_use[' + profile_index_in_unsorted_lists + ']: ' + public_profile_indices_and_types_to_use[profile_index_in_unsorted_lists]);
                            console.debug('========================================');
                        
                            if (public_profile_unique_ids[profile_index_in_unsorted_lists] !== null && public_profile_unique_ids[profile_index_in_unsorted_lists] !== undefined &&
                                public_profile_indices_and_menu_labels_to_use[profile_index_in_unsorted_lists] !== null && public_profile_indices_and_menu_labels_to_use[profile_index_in_unsorted_lists] !== undefined) {
                                console.debug('Adding public profile: ' + public_profile_indices_and_menu_labels_to_use[profile_index_in_unsorted_lists]);
                            
                                var icon_url = window.location.protocol + "//" + window.location.host + img_base_path + '/app_dark.svg';
                                OCA.Files.fileActions.registerAction({
                                    name: public_profile_unique_ids[profile_index_in_unsorted_lists],
                                    displayName: public_profile_indices_and_menu_labels_to_use[profile_index_in_unsorted_lists],
                                    mime: 'application/pdf',
                                    order: -public_profile_priority_values[profile_id_1],
                                    permissions: OC.PERMISSION_UPDATE,
                                    icon: icon_url,
                                    actionHandler: public_callback_functions[profile_index_in_unsorted_lists].bind(public_callback_params[profile_index_in_unsorted_lists])
                                });
                            }
                        }
                    } else {
                        console.debug('Skipping profile_id_1 ' + profile_id_1 + ', not in priority group ' + sorted_public_profile_priority_tiers[priority_group]);
                    }
                }
            }
        }
    }
});
