<?php

/**
* This file is licensed under the Affero General Public License version 3 or
* later. See the COPYING file.
*
* @author Fabian Aichele
* @copyright intarsys GmbH 2021
*/

namespace OCA\IntarsysCloudSuiteGearsIntegration\AppInfo;

use OCP\AppFramework\App;
use OCP\AppFramework\Bootstrap\IBootContext;
use OCP\AppFramework\Bootstrap\IBootstrap;
use OCP\AppFramework\Bootstrap\IRegistrationContext;
use OCP\Notification\IManager as INotificationManager;
use OCA\IntarsysCloudSuiteGearsIntegration\Notification\CloudsuiteGearsNotifier;

use OCA\IntarsysCloudSuiteGearsIntegration\Listener\BeforeTemplateRenderedListener;
use OCP\AppFramework\Http\Events\BeforeTemplateRenderedEvent;

class Application extends App implements IBootstrap {
	private $initial_signature_profile_properties;
	
	private $config;

	public function __construct() {
		parent::__construct('signlive');

	}

	public function register(IRegistrationContext $context): void {
		 // Register the composer autoloader for packages shipped by our app
		 include_once __DIR__ . '/../../vendor/autoload.php';

		// Inject event handlers to add our JavaScript scripts to each PHP page request
		$context->registerEventListener(BeforeTemplateRenderedEvent::class, BeforeTemplateRenderedListener::class);
	}

	public function boot(IBootContext $context): void {
		// Always register our notifier so we can send push notifications on signature jobs progress and errors
		$context->injectFn(function (INotificationManager $notificationsManager) {
			$notificationsManager->registerNotifierService(CloudsuiteGearsNotifier::class);
		});
	}
}
