<?php

/**
 * This file is licensed under the Affero General Public License version 3 or
 * later. See the COPYING file.
 *
 * @author Fabian Aichele
 * @copyright intarsys GmbH 2021
 */

namespace OCA\IntarsysCloudSuiteGearsIntegration\Migration;

use OCP\Migration\IOutput;
use OCP\Migration\IRepairStep;
use Psr\Log\LoggerInterface;
use OCP\IConfig;

use \OCA\IntarsysCloudSuiteGearsIntegration\Util\SignLiveDefaultConfigValues;

class InitialSignatureProfilesRepairStep implements IRepairStep
{

        /** @var LoggerInterface */
        protected $logger; #

        /** @var \OCP\IConfig */
        protected $config;

        public function __construct(
                LoggerInterface $logger,
                IConfig $config
        ) {
                $this->logger = $logger;
                $this->config = $config;
        }

        /**
         * Returns the step's name
         */
        public function getName()
        {
                return 'SignLive! initial signature profiles setup';
        }

        /**
         * @param IOutput $output
         */
        public function run(IOutput $output)
        {
                $output->info('Checking if initial signature profiles for the SignLive! app should be created.');
                $initial_profiles_created = $this->config->getAppValue('signlive', 'cloudsuite_gears_initial_profiles_created', '');
                if ($initial_profiles_created === '') {
                        $output->info('Creating initial SignLive! signature profiles.');
                        foreach (SignLiveDefaultConfigValues::$INITIAL_SIGNATURE_PROFILE_DEFINITIONS as $initial_profile_id => $initial_profile_data) {
                                $unique_profile_key = $initial_profile_data['unique_profile_key'];
                                $this->config->setAppValue(
                                        'signlive',
                                        'cloudsuite_gears_profile_id_' . $unique_profile_key,
                                        $initial_profile_data['cloudsuite_gears_profile_id_' . $unique_profile_key]
                                );
                                $this->config->setAppValue(
                                        'signlive',
                                        'cloudsuite_gears_profile_name_' . $unique_profile_key,
                                        $initial_profile_data['cloudsuite_gears_profile_name_' . $unique_profile_key]
                                );
                                $this->config->setAppValue(
                                        'signlive',
                                        'cloudsuite_gears_profile_type_' . $unique_profile_key,
                                        $initial_profile_data['cloudsuite_gears_profile_type_' . $unique_profile_key]
                                );
                                $this->config->setAppValue(
                                        'signlive',
                                        'cloudsuite_gears_profile_json_' . $unique_profile_key,
                                        $initial_profile_data['cloudsuite_gears_profile_json_' . $unique_profile_key]
                                );
                                $this->config->setAppValue(
                                        'signlive',
                                        'cloudsuite_gears_default_profile_' . $unique_profile_key,
                                        $initial_profile_data['cloudsuite_gears_default_profile_' . $unique_profile_key]
                                );
                                $this->config->setAppValue(
                                        'signlive',
                                        'cloudsuite_gears_profile_nextcloud_groups_' . $unique_profile_key,
                                        $initial_profile_data['cloudsuite_gears_profile_nextcloud_groups_' . $unique_profile_key]
                                );
                                $this->config->setAppValue(
                                        'signlive',
                                        'cloudsuite_gears_profile_menu_priority_' . $unique_profile_key,
                                        $initial_profile_data['cloudsuite_gears_profile_menu_priority_' . $unique_profile_key]
                                );
                        }

                        // Show public signature profiles in web UI by default initially, otherwise there will not be any signature options visible in the web UI
                        // Admins can create restricted profiles after initial installation and switch off this option explicitly
                        $this->config->setAppValue('signlive', 'cloudsuite_gears_show_default_signature_entries_in_context_menu', 'true');

                        // After storing initial profiles indicate that bootstrap is done
                        $this->config->setAppValue('signlive', 'cloudsuite_gears_initial_profiles_created', 'true');
                } else {
                        $output->info('Initial SignLive! signature profiles have already been created previously.');
                }
        }
}
