<?php

/**
* This file is licensed under the Affero General Public License version 3 or
* later. See the COPYING file.
*
* @author Fabian Aichele <fabian.aichele@bpex.de>
* @copyright intarsys AG 2021
*/

namespace OCA\IntarsysCloudSuiteGearsIntegration\Settings;

use OCP\Template;
use OCP\Util;
use OCP\ILogger;
use OCP\IUserSession;
use OCP\AppFramework\Http\TemplateResponse;
use OCP\IConfig;
use OCP\Settings\ISettings;

class Personal implements ISettings {
	/** @var \OCP\IConfig */
	protected $config;

	/** @var IUserSettings */
	protected $userSettings;

	/** @var string */
	protected $user;

	/**
	 * constructor of the controller
	 *
	 * @param IConfig $config
	 * @param IUserSession $userSession
	 * @param IUserManager $userManager
	 */
	public function __construct(IConfig $config,
								IUserSession $userSession) {
		$this->config = $config;
		$this->userSession = $userSession;
        $this->user = $this->userSession->getUser()->getUID();
	}

	/**
	 * @return TemplateResponse
	 */
	public function getForm() {
		\OCP\Util::writeLog('signlive', 'getForm() called in signlive/lib/Settings/Personal.php.', ILogger::DEBUG);

		$user_attribute_title = $this->config->getUserValue($this->user, 'signlive', 'cloudsuite_gears_user_attribute_title', '');
		$user_attribute_given_names = $this->config->getUserValue($this->user, 'signlive', 'cloudsuite_gears_user_attribute_given_names', '');
		$user_attribute_given_surname = $this->config->getUserValue($this->user, 'signlive', 'cloudsuite_gears_user_attribute_surname', '');
		$user_attribute_msisdn = $this->config->getUserValue($this->user, 'signlive', 'cloudsuite_gears_user_attribute_msisdn', '');
		$user_attribute_country = $this->config->getUserValue($this->user, 'signlive', 'cloudsuite_gears_user_attribute_country', 'de');
		$user_attribute_signme_user_name = $this->config->getUserValue($this->user, 'signlive', 'cloudsuite_gears_user_attribute_signme_user_name', '');
		$user_attribute_encoded_signature_image = $this->config->getUserValue($this->user, 'signlive', 'cloudsuite_gears_user_attribute_encoded_signature_image', '');
		$user_attribute_company_division = $this->config->getUserValue($this->user, 'signlive', 'cloudsuite_gears_user_attribute_company_division', '');

		$parameters = [
			'cloudsuite_gears_num_user_settings' => $num_user_signature_profiles,
			'cloudsuite_gears_user_attribute_title' => $user_attribute_title,
			'cloudsuite_gears_user_attribute_given_names' => $user_attribute_given_names,
			'cloudsuite_gears_user_attribute_surname' => $user_attribute_given_surname,
			'cloudsuite_gears_user_attribute_msisdn' => $user_attribute_msisdn,
			'cloudsuite_gears_user_attribute_country' => $user_attribute_country,
			'cloudsuite_gears_user_attribute_signme_user_name' => $user_attribute_signme_user_name,
			'cloudsuite_gears_user_attribute_encoded_signature_image' => $user_attribute_encoded_signature_image,
			'cloudsuite_gears_user_attribute_company_division' => $user_attribute_company_division,
		];

		return new TemplateResponse('signlive', 'personal_settings', $parameters);
	}

	/**
	 * @return string the section ID, e.g. 'sharing'
	 */
	public function getSection() {
		return 'signlive';
	}

	/**
	 * @return int whether the form should be rather on the top or bottom of
	 * the admin section. The forms are arranged in ascending order of the
	 * priority values. It is required to return a value between 0 and 100.
	 *
	 * E.g.: 70
	 */
	public function getPriority() {
		return 55;
	}
}
