<?php

/**
* This file is licensed under the Affero General Public License version 3 or
* later. See the COPYING file.
*
* @author Fabian Aichele <fabian.aichele@bpex.de>
* @copyright intarsys AG 2021
*/

namespace OCA\IntarsysCloudSuiteGearsIntegration\Util;

use OCP\Util;
use OCP\ILogger;
use OCP\IConfig;
use OCP\IUserSession;
use OCP\Files\IRootFolder;
use OCP\Files\IMimeTypeDetector;
use OCP\Files\NotFoundException;
use OCP\Files\NotPermittedException;
use OCP\Files\File;
use OCP\Files\Folder;
use OCP\AppFramework\Utility\ITimeFactory;

class FileIO {
    /** @var \OCP\IConfig */
    protected $config;

    /** @var \OCP\Files\IMimeTypeDetector */
    protected $mimeTypeDetector;

    /** @var \OCP\Files\IRootFolder */
    protected $storage;

    /** @var ITimeFactory */
	protected $timeFactory;

    /** @var IUserSession */
    protected $userSession;

    public function __construct(IConfig $config,
                                IMimeTypeDetector $mimeTypeDetector,
                                IRootFolder $storage,
                                IUserSession $userSession,
                                ITimeFactory $timeFactory) {
        $this->config = $config;
        $this->mimeTypeDetector = $mimeTypeDetector;
        $this->storage = $storage;
        $this->userSession = $userSession;
        $this->timeFactory = $timeFactory;
    }

    public function getBase64EncodedFile($file_name) {
        \OCP\Util::writeLog('signlive', 'getBase64EncodedFile('.$file_name.')', ILogger::DEBUG);
        $current_user_folder = $this->storage->getUserFolder($this->userSession->getUser()->getUID());
        $encoded_file = '';
        try {
            try {
                \OCP\Util::writeLog('signlive', 'Reading file: '.$file_name, ILogger::DEBUG);
                $file_handle = $current_user_folder->get($file_name);
                if ($file_handle instanceof File) {
                    \OCP\Util::writeLog('signlive', 'Entry is a regular file: '.$file_name, ILogger::DEBUG);
                    $file_content = $file_handle->getContent();
                    $encoded_file = base64_encode($file_content);
                } else {
                    \OCP\Util::writeLog('signlive', 'Entry is NOT a regular file, can not proceed: '.$file_name, ILogger::WARN);
                }
            } catch (OCP\Files\NotFoundException $e) {  
                \OCP\Util::writeLog('signlive', 'File not found: '.$file_to_sign.', can not proceed!', ILogger::WARN);
                $encoded_file = '';
            }
        } catch (OCP\Files\NotPermittedException $e) {
            \OCP\Util::writeLog('signlive', 'No permissions to read file '.$file_to_sign.' , can not proceed!', ILogger::WARN);
            $encoded_file = '';
        }
        return $encoded_file;
    }
}