<?php

/**
* This file is licensed under the Affero General Public License version 3 or
* later. See the COPYING file.
*
* @author Fabian Aichele <fabian.aichele@bpex.de>
* @copyright intarsys AG 2021
*/

namespace OCA\IntarsysCloudSuiteGearsIntegration\Util;

class SignLiveDefaultConfigValues {
    public static $GEARS_SERVER_PROTOCOL = 'https';
    public static $GEARS_SERVER_HOST = 'cloudsuite.intarsys.de';
    public static $GEARS_SERVER_PORT = '443';
    public static $GEARS_SERVER_CORE_PATH = '/cloudsuite-gears-nextcloud/core';

    public static $DEFAULT_GLOBAL_ATTRIBUTE_COMPANY_NAME = 'Sign City AG';

    public static $SHOW_PUBLIC_PROFILES_IN_UI_MENU = 1;
    public static $DEFAULT_MENU_PRIORITY_VALUE = 75;
    public static $SHOW_PRIORITY_VALUES_IN_MENU = 0;
    public static $NOTIFICATION_DISPLAY_DURATION = 15;
    public static $NOTIFICATION_CHECK_INTERVAL = 2;
    public static $SHOW_NOTIFICATIONS_FOR_SUCCESSFUL_SIGNATURE_REQUESTS = 0;
    public static $SHOW_PERSISTENT_NOTIFICATIONS_FOR_FAILED_SIGNATURE_REQUESTS = 0;
    public static $USE_UNIFIED_JOB_CONTROL = 0;

    // Initial signature profile definitions
    public static $INITIAL_SIGNATURE_PROFILE_DEFINITIONS = [
        'cloudsuite_gears_profile_0;;Default profile 1' => [
            'unique_profile_key' => '0;;Default profile 1',
            'cloudsuite_gears_profile_id_0;;Default profile 1' => '0;;Default profile 1',
            'cloudsuite_gears_profile_name_0;;Default profile 1' => 'View & Sign (D-Trust)',
            'cloudsuite_gears_profile_type_0;;Default profile 1' => 'viewer_based_signature',
            'cloudsuite_gears_default_profile_0;;Default profile 1' => 1,
            'cloudsuite_gears_profile_json_0;;Default profile 1' => "{
                        \"variables\": {
                \"signerCreate.configuration.canvas\": \"ncSealSignMe\",
                \"signerCreate.configuration.pencil\": \"ncSignMe\",
                \"signerCreate.configuration.signatureImage\": \"ncSealSignMeWithSignatureImage\",
                \"signerCreate.configuration.invisible\": \"ncSignMe\",
                \"signerCreate.configuration.fixedPosition\": \"ncSealSignMeFixedPosition\",
                                \"bridge.principalFilter\": \"true\",
                                \"off.bridge.principalFilter\": \"principal.supportsSignature && principal.qualified\",
                                \"bridge.deviceProviderInclude\": \"demo;pkcs11;smartcard;windows\",
                                \"signatureImage\": \"\${sl.encodedsignatureimage}\"
                        },
                        \"options\": {
                                \"redirectUri\": \"\${sl.system.redirect_url}\",
                                \"principal\": {
                                        \"name\": \"\${nc.displayname}\",
                                        \"claims\": {
                                                \"msisdn\": \"\${sl.msisdn}\",
                                                \"givenname\": \"\${sl.givenname}\",
                                                \"surname\": \"\${sl.surname}\",
                                                \"c\": \"\${sl.country}\",
                                                \"signMeUsername\": \"\${sl.signmeusername}\"
                                        }
                                }
                        },
                        \"configuration\": \"ncSignature\",
                        \${sl.system.documents_section}
                }",
            'cloudsuite_gears_profile_nextcloud_groups_0;;Default profile 1' => '',
            'cloudsuite_gears_profile_menu_priority_0;;Default profile 1' => 75
        ],
        'cloudsuite_gears_profile_1;;Default profile 2' => [
            'unique_profile_key' => '1;;Default profile 2',
            'cloudsuite_gears_profile_id_1;;Default profile 2' => '1;;Default profile 2',
            'cloudsuite_gears_profile_name_1;;Default profile 2' => 'Seal (D-Trust QSeal ID)',
            'cloudsuite_gears_profile_type_1;;Default profile 2' => 'automatic_signature',
            'cloudsuite_gears_default_profile_1;;Default profile 2' => 1,
            'cloudsuite_gears_profile_json_1;;Default profile 2' => "{
                        \"variables\": {
                                \"signatureImage\": \"\${sl.encodedsignatureimage}\"
                        },
                        \"options\": {
                                \"redirectUri\": \"\${sl.system.redirect_url}\",
                                \"principal\": {
                                        \"name\": \"\${nc.displayname}\",
                                        \"claims\": {
                                                \"msisdn\": \"\${sl.msisdn}\",
                                                \"givenname\": \"\${sl.givenname}\",
                                                \"surname\": \"\${sl.surname}\",
                                                \"c\": \"\${sl.country}\",
                                                \"signMeUsername\": \"\${sl.signmeusername}\"
                                        }
                                }
                        },
                        \"configuration\": \"ncSealSignMeFixedPosition\",
                        \${sl.system.documents_section}
                }",
            'cloudsuite_gears_profile_nextcloud_groups_1;;Default profile 2' => '',
            'cloudsuite_gears_profile_menu_priority_1;;Default profile 2' => 75
        ],
        'cloudsuite_gears_profile_2;;View & Sign' => [
            'unique_profile_key' => '2;;View & Sign',
            'cloudsuite_gears_profile_id_2;;View & Sign' => '2;;View & Sign',
            'cloudsuite_gears_profile_name_2;;View & Sign' => 'View & Sign (Swisscom AIS)',
            'cloudsuite_gears_profile_type_2;;View & Sign' => 'viewer_based_signature',
            'cloudsuite_gears_default_profile_2;;View & Sign' => 1,
            'cloudsuite_gears_profile_json_2;;View & Sign' => "
                {
                        \"variables\": {
                \"signerCreate.configuration.canvas\": \"ncSealAis\",
                \"signerCreate.configuration.pencil\": \"aisStatic\",
                \"signerCreate.configuration.signatureImage\": \"ncSealAisWithSignatureImage\",
                \"signerCreate.configuration.invisible\": \"aisStatic\",
                \"signerCreate.configuration.fixedPosition\": \"ncSealAisFixedPosition\",
                                \"bridge.principalFilter\": \"true\",
                                \"off.bridge.principalFilter\": \"principal.supportsSignature && principal.qualified\",
                                \"bridge.deviceProviderInclude\": \"demo;pkcs11;smartcard;windows\",
                                \"signatureImage\": \"\${sl.encodedsignatureimage}\"
                        },
                        \"options\": {
                                \"redirectUri\": \"\${sl.system.redirect_url}\",
                                \"principal\": {
                                        \"name\": \"\${nc.displayname}\",
                                        \"claims\": {
                                                \"msisdn\": \"\${sl.msisdn}\",
                                                \"givenname\": \"\${sl.givenname}\",
                                                \"surname\": \"\${sl.surname}\",
                                                \"c\": \"\${sl.country}\",
                                                \"signMeUsername\": \"\${sl.signmeusername}\"
                                        }
                                }
                        },
                        \"configuration\": \"ncSignature\",
                        \${sl.system.documents_section}
                }",
            'cloudsuite_gears_profile_nextcloud_groups_2;;View & Sign' => '',
            'cloudsuite_gears_profile_menu_priority_2;;View & Sign' => 75
        ],
        'cloudsuite_gears_profile_3;;Sign' => [
            'unique_profile_key' => '3;;Sign',
            'cloudsuite_gears_profile_id_3;;Sign' => '3;;Sign',
            'cloudsuite_gears_profile_name_3;;Sign' => 'Seal (Swisscom AIS)',
            'cloudsuite_gears_profile_type_3;;Sign' => 'automatic_signature',
            'cloudsuite_gears_default_profile_3;;Sign' => 1,
            'cloudsuite_gears_profile_json_3;;Sign' => "{
                        \"variables\": {
                                \"signatureImage\": \"\${sl.encodedsignatureimage}\"
                        },
                        \"options\": {
                                \"redirectUri\": \"\${sl.system.redirect_url}\",
                                \"principal\": {
                                        \"name\": \"\${nc.displayname}\",
                                        \"claims\": {
                                                \"msisdn\": \"\${sl.msisdn}\",
                                                \"givenname\": \"\${sl.givenname}\",
                                                \"surname\": \"\${sl.surname}\",
                                                \"c\": \"\${sl.country}\",
                                                \"signMeUsername\": \"\${sl.signmeusername}\"
                                        }
                                }
                        },
                        \"configuration\": \"ncSealAisFixedPosition\",
                        \${sl.system.documents_section}
                }",
            'cloudsuite_gears_profile_nextcloud_groups_3;;Sign' => '',
            'cloudsuite_gears_profile_menu_priority_3;;Sign' => 75
        ],
        'cloudsuite_gears_profile_4;;Sign on mobile' => [
            'unique_profile_key' => '4;;Sign on mobile',
            'cloudsuite_gears_profile_id_4;;Sign on mobile' => '4;;Sign on mobile',
            'cloudsuite_gears_profile_name_4;;Sign on mobile' => 'Sign on Mobile',
            'cloudsuite_gears_profile_type_4;;Sign on mobile' => 'viewer_based_signature',
            'cloudsuite_gears_default_profile_4;;Sign on mobile' => 1,
            'cloudsuite_gears_profile_json_4;;Sign on mobile' => "{
                        \"variables\": {
                                \"bridge.principalFilter\": \"true\",
                                \"off.bridge.principalFilter\": \"principal.supportsSignature && principal.qualified\",
                                \"bridge.deviceProviderInclude\": \"demo;pkcs11;smartcard;windows\",
                                \"signatureImage\": \"\${sl.encodedsignatureimage}\"
                        },
                        \"options\": {
                                \"redirectUri\": \"\${sl.system.redirect_url}\",
                                \"principal\": {
                                        \"name\": \"\${nc.displayname}\",
                                        \"claims\": {
                                                \"msisdn\": \"\${sl.msisdn}\",
                                                \"givenname\": \"\${sl.givenname}\",
                                                \"surname\": \"\${sl.surname}\",
                                                \"c\": \"\${sl.country}\",
                                                \"signMeUsername\": \"\${sl.signmeusername}\"
                                        }
                                }
                        },
                        \"configuration\": \"ncSmartphone\",
                        \${sl.system.documents_section}
                }",
            'cloudsuite_gears_profile_nextcloud_groups_4;;Sign on mobile' => '',
            'cloudsuite_gears_profile_menu_priority_4;;Sign on mobile' => 75
        ]
    ];
}
