<?php

/**
 * This file is licensed under the Affero General Public License version 3 or
 * later. See the COPYING file.
 *
 * @author Fabian Aichele <fabian.aichele@bpex.de>
 * @copyright intarsys AG 2021
 */

namespace OCA\IntarsysCloudSuiteGearsIntegration\Util;

use OCP\Util;
use OCP\ILogger;
use OCP\IConfig;

use OCA\IntarsysCloudSuiteGearsIntegration\Util\SignLiveDefaultConfigValues;

class SignatureProfilesSetup
{
    /** @var \OCP\IConfig */
    protected $config;

    public function __construct(IConfig $config)
    {
        $this->config = $config;
    }

    /**
     * (Re)create initial SignLive! signature profile definitions
     * @return bool
     */
    public function setupDefaultSignatureProfiles() {
        \OCP\Util::writeLog('signlive', 'setupDefaultSignatureProfiles() called in signlive/lib/Util/SignatureProfilesSetup.php.', ILogger::DEBUG);
        $initial_profiles_created = $this->config->getAppValue('signlive', 'cloudsuite_gears_initial_profiles_created', '');
        \OCP\Util::writeLog('signlive', 'cloudsuite_gears_initial_profiles_created = '.$initial_profiles_created, ILogger::DEBUG);
        if ($initial_profiles_created === '') {
            \OCP\Util::writeLog('signlive', 'Creating initial SignLive! signature profiles.', ILogger::INFO);
            foreach (SignLiveDefaultConfigValues::$INITIAL_SIGNATURE_PROFILE_DEFINITIONS as $initial_profile_id => $initial_profile_data) {
                    $unique_profile_key = $initial_profile_data['unique_profile_key'];
                    \OCP\Util::writeLog('signlive', 'Creating signature profile: '.$initial_profile_id, ILogger::INFO);
                    $this->config->setAppValue(
                            'signlive',
                            'cloudsuite_gears_profile_id_' . $unique_profile_key,
                            $initial_profile_data['cloudsuite_gears_profile_id_' . $unique_profile_key]
                    );
                    $this->config->setAppValue(
                            'signlive',
                            'cloudsuite_gears_profile_name_' . $unique_profile_key,
                            $initial_profile_data['cloudsuite_gears_profile_name_' . $unique_profile_key]
                    );
                    $this->config->setAppValue(
                            'signlive',
                            'cloudsuite_gears_profile_type_' . $unique_profile_key,
                            $initial_profile_data['cloudsuite_gears_profile_type_' . $unique_profile_key]
                    );
                    $this->config->setAppValue(
                            'signlive',
                            'cloudsuite_gears_profile_json_' . $unique_profile_key,
                            $initial_profile_data['cloudsuite_gears_profile_json_' . $unique_profile_key]
                    );
                    $this->config->setAppValue(
                            'signlive',
                            'cloudsuite_gears_default_profile_' . $unique_profile_key,
                            $initial_profile_data['cloudsuite_gears_default_profile_' . $unique_profile_key]
                    );
                    $this->config->setAppValue(
                            'signlive',
                            'cloudsuite_gears_profile_nextcloud_groups_' . $unique_profile_key,
                            $initial_profile_data['cloudsuite_gears_profile_nextcloud_groups_' . $unique_profile_key]
                    );
                    $this->config->setAppValue(
                            'signlive',
                            'cloudsuite_gears_profile_menu_priority_' . $unique_profile_key,
                            $initial_profile_data['cloudsuite_gears_profile_menu_priority_' . $unique_profile_key]
                    );
            }

            // Show public signature profiles in web UI by default initially, otherwise there will not be any signature options visible in the web UI
            // Admins can create restricted profiles after initial installation and switch off this option explicitly
            $this->config->setAppValue('signlive', 'cloudsuite_gears_show_default_signature_entries_in_context_menu', 'true');

            // After storing initial profiles indicate that bootstrap is done
            $this->config->setAppValue('signlive', 'cloudsuite_gears_initial_profiles_created', 'true');
            return true;
        } else {
            \OCP\Util::writeLog('signlive', 'Initial SignLive! signature profiles have already been created previously.', ILogger::DEBUG);
            return false;
        }    
    }
}
